/* Generated by Cython 0.29.15 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_15"
#define CYTHON_HEX_VERSION 0x001D0FF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__bgzip_utils
#define __PYX_HAVE_API__bgzip_utils
/* Early includes */
#include <string.h>
#include <stdlib.h>
#include "zlib.h"
#include <stdio.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "bgzip_utils/bgzip_utils.pyx",
  "type.pxd",
};
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()


/* "czlib.pxd":1
 * ctypedef unsigned char Bytef             # <<<<<<<<<<<<<<
 * ctypedef unsigned long uLongf
 * ctypedef unsigned int uInt
 */
typedef unsigned char __pyx_t_5czlib_Bytef;

/* "czlib.pxd":2
 * ctypedef unsigned char Bytef
 * ctypedef unsigned long uLongf             # <<<<<<<<<<<<<<
 * ctypedef unsigned int uInt
 * ctypedef z_stream_s z_stream
 */
typedef unsigned long __pyx_t_5czlib_uLongf;

/* "czlib.pxd":3
 * ctypedef unsigned char Bytef
 * ctypedef unsigned long uLongf
 * ctypedef unsigned int uInt             # <<<<<<<<<<<<<<
 * ctypedef z_stream_s z_stream
 * 
 */
typedef unsigned int __pyx_t_5czlib_uInt;

/*--- Type declarations ---*/

/* "czlib.pxd":4
 * ctypedef unsigned long uLongf
 * ctypedef unsigned int uInt
 * ctypedef z_stream_s z_stream             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "zlib.h":
 */
typedef struct z_stream_s __pyx_t_5czlib_z_stream;
struct __pyx_t_11bgzip_utils_block_header_s;
struct __pyx_t_11bgzip_utils_block_header_subfield_s;
struct __pyx_t_11bgzip_utils_block_header_bgzip_subfield_s;
struct __pyx_t_11bgzip_utils_block_tailer_s;
struct __pyx_t_11bgzip_utils_block_s;
struct __pyx_t_11bgzip_utils_bgzip_stream_s;

/* "bgzip_utils.pyx":13
 * 
 * 
 * cdef enum:             # <<<<<<<<<<<<<<
 *     NUMBER_OF_BLOCKS = 20000
 *     MAGIC_LENGTH = 4
 */
enum  {
  __pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS = 0x4E20,
  __pyx_e_11bgzip_utils_MAGIC_LENGTH = 4,
  __pyx_e_11bgzip_utils_BGZIP_OK = 0,
  __pyx_e_11bgzip_utils_BGZIP_INSUFFICIENT_BYTES = -1L
};

/* "bgzip_utils.pyx":25
 * 
 * 
 * ctypedef block_header_s BlockHeader             # <<<<<<<<<<<<<<
 * cdef struct block_header_s:
 *     unsigned char magic[MAGIC_LENGTH]
 */
typedef struct __pyx_t_11bgzip_utils_block_header_s __pyx_t_11bgzip_utils_BlockHeader;

/* "bgzip_utils.pyx":26
 * 
 * ctypedef block_header_s BlockHeader
 * cdef struct block_header_s:             # <<<<<<<<<<<<<<
 *     unsigned char magic[MAGIC_LENGTH]
 *     unsigned int mod_time
 */
struct __pyx_t_11bgzip_utils_block_header_s {
  unsigned char magic[__pyx_e_11bgzip_utils_MAGIC_LENGTH];
  unsigned int mod_time;
  unsigned char extra_flags;
  unsigned char os_type;
  unsigned short extra_len;
};

/* "bgzip_utils.pyx":34
 * 
 * 
 * ctypedef block_header_subfield_s BlockHeaderSubfield             # <<<<<<<<<<<<<<
 * cdef struct block_header_subfield_s:
 *     unsigned char id_[2]
 */
typedef struct __pyx_t_11bgzip_utils_block_header_subfield_s __pyx_t_11bgzip_utils_BlockHeaderSubfield;

/* "bgzip_utils.pyx":35
 * 
 * ctypedef block_header_subfield_s BlockHeaderSubfield
 * cdef struct block_header_subfield_s:             # <<<<<<<<<<<<<<
 *     unsigned char id_[2]
 *     unsigned short length
 */
struct __pyx_t_11bgzip_utils_block_header_subfield_s {
  unsigned char id_[2];
  unsigned short length;
};

/* "bgzip_utils.pyx":40
 * 
 * 
 * ctypedef block_header_bgzip_subfield_s BlockHeaderBGZipSubfield             # <<<<<<<<<<<<<<
 * cdef struct block_header_bgzip_subfield_s:
 *     unsigned char id_[2]
 */
typedef struct __pyx_t_11bgzip_utils_block_header_bgzip_subfield_s __pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield;

/* "bgzip_utils.pyx":41
 * 
 * ctypedef block_header_bgzip_subfield_s BlockHeaderBGZipSubfield
 * cdef struct block_header_bgzip_subfield_s:             # <<<<<<<<<<<<<<
 *     unsigned char id_[2]
 *     unsigned short length
 */
struct __pyx_t_11bgzip_utils_block_header_bgzip_subfield_s {
  unsigned char id_[2];
  unsigned short length;
  unsigned short block_size;
};

/* "bgzip_utils.pyx":47
 * 
 * 
 * ctypedef block_tailer_s BlockTailer             # <<<<<<<<<<<<<<
 * cdef struct block_tailer_s:
 *     unsigned int crc
 */
typedef struct __pyx_t_11bgzip_utils_block_tailer_s __pyx_t_11bgzip_utils_BlockTailer;

/* "bgzip_utils.pyx":48
 * 
 * ctypedef block_tailer_s BlockTailer
 * cdef struct block_tailer_s:             # <<<<<<<<<<<<<<
 *     unsigned int crc
 *     unsigned int inflated_size
 */
struct __pyx_t_11bgzip_utils_block_tailer_s {
  unsigned int crc;
  unsigned int inflated_size;
};

/* "bgzip_utils.pyx":53
 * 
 * 
 * ctypedef block_s Block             # <<<<<<<<<<<<<<
 * cdef struct block_s:
 *     unsigned int deflated_size
 */
typedef struct __pyx_t_11bgzip_utils_block_s __pyx_t_11bgzip_utils_Block;

/* "bgzip_utils.pyx":54
 * 
 * ctypedef block_s Block
 * cdef struct block_s:             # <<<<<<<<<<<<<<
 *     unsigned int deflated_size
 *     unsigned int inflated_size
 */
struct __pyx_t_11bgzip_utils_block_s {
  unsigned int deflated_size;
  unsigned int inflated_size;
  unsigned int crc;
  unsigned short block_size;
  __pyx_t_5czlib_Bytef *next_in;
  unsigned int available_in;
  __pyx_t_5czlib_Bytef *next_out;
  unsigned int avail_out;
};

/* "bgzip_utils.pyx":65
 * 
 * 
 * ctypedef bgzip_stream_s BGZipStream             # <<<<<<<<<<<<<<
 * cdef struct bgzip_stream_s:
 *     unsigned int available_in
 */
typedef struct __pyx_t_11bgzip_utils_bgzip_stream_s __pyx_t_11bgzip_utils_BGZipStream;

/* "bgzip_utils.pyx":66
 * 
 * ctypedef bgzip_stream_s BGZipStream
 * cdef struct bgzip_stream_s:             # <<<<<<<<<<<<<<
 *     unsigned int available_in
 *     Bytef *next_in
 */
struct __pyx_t_11bgzip_utils_bgzip_stream_s {
  unsigned int available_in;
  __pyx_t_5czlib_Bytef *next_in;
};

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* IncludeStringH.proto */
#include <string.h>

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'czlib' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython_nogil' */

/* Module declarations from 'bgzip_utils' */
static unsigned char const *__pyx_v_11bgzip_utils_MAGIC;
static PyObject *__pyx_v_11bgzip_utils_HEADER = 0;
static unsigned int __pyx_v_11bgzip_utils__block_inflated_size;
static unsigned int __pyx_v_11bgzip_utils__block_metadata_size;
static void __pyx_f_11bgzip_utils_c_exception(char const *); /*proto*/
static void __pyx_f_11bgzip_utils_inflate_block(__pyx_t_11bgzip_utils_Block *); /*proto*/
static void *__pyx_f_11bgzip_utils_ref_and_advance(__pyx_t_11bgzip_utils_BGZipStream *, unsigned int, int *); /*proto*/
static int __pyx_f_11bgzip_utils_read_block(__pyx_t_11bgzip_utils_Block *, __pyx_t_11bgzip_utils_BGZipStream *); /*proto*/
static void __pyx_f_11bgzip_utils_compress_block(__pyx_t_11bgzip_utils_Block *); /*proto*/
static void __pyx_f_11bgzip_utils__get_buffer(PyObject *, Py_buffer *); /*proto*/
#define __Pyx_MODULE_NAME "bgzip_utils"
extern int __pyx_module_is_main_bgzip_utils;
int __pyx_module_is_main_bgzip_utils = 0;

/* Implementation of 'bgzip_utils' */
static PyObject *__pyx_builtin_range;
static const char __pyx_k_i[] = "i";
static const char __pyx_k_BC[] = "\037\213\010\004\000\000\000\000\000\377\006\000BC\002\000";
static const char __pyx_k_io[] = "io";
static const char __pyx_k_dst[] = "dst";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_out[] = "out";
static const char __pyx_k_ret[] = "ret";
static const char __pyx_k_src[] = "src";
static const char __pyx_k_ceil[] = "ceil";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_zlib[] = "zlib";
static const char __pyx_k_chunk[] = "chunk";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_blocks[] = "blocks";
static const char __pyx_k_chunks[] = "chunks";
static const char __pyx_k_handle[] = "handle";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_dst_buff[] = "dst_buff";
static const char __pyx_k_src_buff[] = "src_buff";
static const char __pyx_k_avail_out[] = "avail_out";
static const char __pyx_k_bytes_read[] = "bytes_read";
static const char __pyx_k_chunk_size[] = "chunk_size";
static const char __pyx_k_input_view[] = "input_view";
static const char __pyx_k_bgzip_utils[] = "bgzip_utils";
static const char __pyx_k_num_threads[] = "num_threads";
static const char __pyx_k_dst_buff_obj[] = "dst_buff_obj";
static const char __pyx_k_input_chunks[] = "input_chunks";
static const char __pyx_k_inflated_size[] = "inflated_size";
static const char __pyx_k_bytes_inflated[] = "bytes_inflated";
static const char __pyx_k_input_buff_obj[] = "input_buff_obj";
static const char __pyx_k_bytes_available[] = "bytes_available";
static const char __pyx_k_compress_chunks[] = "compress_chunks";
static const char __pyx_k_decompress_into[] = "decompress_into";
static const char __pyx_k_scratch_buffers[] = "scratch_buffers";
static const char __pyx_k_compressed_chunk[] = "compressed_chunk";
static const char __pyx_k_number_of_blocks[] = "number_of_blocks";
static const char __pyx_k_number_of_chunks[] = "number_of_chunks";
static const char __pyx_k_compressed_chunks[] = "compressed_chunks";
static const char __pyx_k_decompress_into_2[] = "decompress_into_2";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_compress_to_stream[] = "compress_to_stream";
static const char __pyx_k_block_inflated_size[] = "block_inflated_size";
static const char __pyx_k_block_metadata_size[] = "block_metadata_size";
static const char __pyx_k_chunks_per_call_Received[] = " chunks per call. Received ";
static const char __pyx_k_Cannot_compress_more_than[] = "Cannot compress more than ";
static const char __pyx_k_bgzip_utils_bgzip_utils_pyx[] = "bgzip_utils/bgzip_utils.pyx";
static const char __pyx_k_compressed_chunk_guess_size[] = "compressed_chunk_guess_size";
static const char __pyx_k_dst_buff_obj_must_be_a_memoryvie[] = "dst_buff_obj must be a memoryview instance.";
static PyObject *__pyx_kp_b_BC;
static PyObject *__pyx_kp_u_Cannot_compress_more_than;
static PyObject *__pyx_n_s_avail_out;
static PyObject *__pyx_n_s_bgzip_utils;
static PyObject *__pyx_kp_s_bgzip_utils_bgzip_utils_pyx;
static PyObject *__pyx_n_s_block_inflated_size;
static PyObject *__pyx_n_s_block_metadata_size;
static PyObject *__pyx_n_s_blocks;
static PyObject *__pyx_n_s_bytes_available;
static PyObject *__pyx_n_s_bytes_inflated;
static PyObject *__pyx_n_s_bytes_read;
static PyObject *__pyx_n_s_ceil;
static PyObject *__pyx_n_s_chunk;
static PyObject *__pyx_n_s_chunk_size;
static PyObject *__pyx_n_s_chunks;
static PyObject *__pyx_kp_u_chunks_per_call_Received;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_compress_chunks;
static PyObject *__pyx_n_s_compress_to_stream;
static PyObject *__pyx_n_s_compressed_chunk;
static PyObject *__pyx_n_s_compressed_chunk_guess_size;
static PyObject *__pyx_n_s_compressed_chunks;
static PyObject *__pyx_n_s_decompress_into;
static PyObject *__pyx_n_s_decompress_into_2;
static PyObject *__pyx_n_s_dst;
static PyObject *__pyx_n_s_dst_buff;
static PyObject *__pyx_n_s_dst_buff_obj;
static PyObject *__pyx_kp_s_dst_buff_obj_must_be_a_memoryvie;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_handle;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_inflated_size;
static PyObject *__pyx_n_s_input_buff_obj;
static PyObject *__pyx_n_s_input_chunks;
static PyObject *__pyx_n_s_input_view;
static PyObject *__pyx_n_s_io;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_math;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_num_threads;
static PyObject *__pyx_n_s_number_of_blocks;
static PyObject *__pyx_n_s_number_of_chunks;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_out;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_ret;
static PyObject *__pyx_n_s_scratch_buffers;
static PyObject *__pyx_n_s_src;
static PyObject *__pyx_n_s_src_buff;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_write;
static PyObject *__pyx_n_s_zlib;
static PyObject *__pyx_pf_11bgzip_utils_decompress_into(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_src_buff, PyObject *__pyx_v_dst_buff, CYTHON_UNUSED int __pyx_v_num_threads); /* proto */
static PyObject *__pyx_pf_11bgzip_utils_2decompress_into_2(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_src_buff, PyObject *__pyx_v_dst_buff_obj, unsigned int __pyx_v_offset, CYTHON_UNUSED int __pyx_v_num_threads); /* proto */
static PyObject *__pyx_pf_11bgzip_utils_4compress_chunks(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_input_chunks, CYTHON_UNUSED int __pyx_v_num_threads); /* proto */
static PyObject *__pyx_pf_11bgzip_utils_6compress_to_stream(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_input_buff_obj, PyObject *__pyx_v_scratch_buffers, PyObject *__pyx_v_handle, CYTHON_UNUSED int __pyx_v_num_threads); /* proto */
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_codeobj__3;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
/* Late includes */

/* "bgzip_utils.pyx":71
 * 
 * 
 * cdef void c_exception(const char * reason) nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         print(reason)
 */

static void __pyx_f_11bgzip_utils_c_exception(char const *__pyx_v_reason) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("c_exception", 1);

  /* "bgzip_utils.pyx":72
 * 
 * cdef void c_exception(const char * reason) nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         print(reason)
 *         abort()
 */
  /*try:*/ {
    {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        /*try:*/ {

          /* "bgzip_utils.pyx":73
 * cdef void c_exception(const char * reason) nogil:
 *     with gil:
 *         print(reason)             # <<<<<<<<<<<<<<
 *         abort()
 * 
 */
          __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_reason); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          if (__Pyx_PrintOne(0, __pyx_t_1) < 0) __PYX_ERR(0, 73, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "bgzip_utils.pyx":74
 *     with gil:
 *         print(reason)
 *         abort()             # <<<<<<<<<<<<<<
 * 
 * 
 */
          abort();
        }

        /* "bgzip_utils.pyx":72
 * 
 * cdef void c_exception(const char * reason) nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         print(reason)
 *         abort()
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L8;
          }
          __pyx_L7_error: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L4_error;
          }
          __pyx_L8:;
        }
    }
  }
  /*finally:*/ {
    /*normal exit:*/{
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L5;
    }
    __pyx_L4_error: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "bgzip_utils.pyx":71
 * 
 * 
 * cdef void c_exception(const char * reason) nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         print(reason)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("bgzip_utils.c_exception", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_L0:;
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "bgzip_utils.pyx":77
 * 
 * 
 * cdef void inflate_block(Block * block) nogil:             # <<<<<<<<<<<<<<
 *     cdef z_stream zst
 *     cdef int err
 */

static void __pyx_f_11bgzip_utils_inflate_block(__pyx_t_11bgzip_utils_Block *__pyx_v_block) {
  __pyx_t_5czlib_z_stream __pyx_v_zst;
  int __pyx_v_err;
  unsigned int __pyx_t_1;
  __pyx_t_5czlib_Bytef *__pyx_t_2;
  int __pyx_t_3;

  /* "bgzip_utils.pyx":81
 *     cdef int err
 * 
 *     zst.zalloc = NULL             # <<<<<<<<<<<<<<
 *     zst.zfree = NULL
 *     zst.opaque = NULL
 */
  __pyx_v_zst.zalloc = NULL;

  /* "bgzip_utils.pyx":82
 * 
 *     zst.zalloc = NULL
 *     zst.zfree = NULL             # <<<<<<<<<<<<<<
 *     zst.opaque = NULL
 *     zst.avail_in = block.deflated_size
 */
  __pyx_v_zst.zfree = NULL;

  /* "bgzip_utils.pyx":83
 *     zst.zalloc = NULL
 *     zst.zfree = NULL
 *     zst.opaque = NULL             # <<<<<<<<<<<<<<
 *     zst.avail_in = block.deflated_size
 *     zst.avail_out = 1024 * 1024
 */
  __pyx_v_zst.opaque = NULL;

  /* "bgzip_utils.pyx":84
 *     zst.zfree = NULL
 *     zst.opaque = NULL
 *     zst.avail_in = block.deflated_size             # <<<<<<<<<<<<<<
 *     zst.avail_out = 1024 * 1024
 *     zst.next_in = block.next_in
 */
  __pyx_t_1 = __pyx_v_block->deflated_size;
  __pyx_v_zst.avail_in = __pyx_t_1;

  /* "bgzip_utils.pyx":85
 *     zst.opaque = NULL
 *     zst.avail_in = block.deflated_size
 *     zst.avail_out = 1024 * 1024             # <<<<<<<<<<<<<<
 *     zst.next_in = block.next_in
 *     zst.next_out = block.next_out
 */
  __pyx_v_zst.avail_out = 0x100000;

  /* "bgzip_utils.pyx":86
 *     zst.avail_in = block.deflated_size
 *     zst.avail_out = 1024 * 1024
 *     zst.next_in = block.next_in             # <<<<<<<<<<<<<<
 *     zst.next_out = block.next_out
 * 
 */
  __pyx_t_2 = __pyx_v_block->next_in;
  __pyx_v_zst.next_in = __pyx_t_2;

  /* "bgzip_utils.pyx":87
 *     zst.avail_out = 1024 * 1024
 *     zst.next_in = block.next_in
 *     zst.next_out = block.next_out             # <<<<<<<<<<<<<<
 * 
 *     err = inflateInit2(&zst, -15)
 */
  __pyx_t_2 = __pyx_v_block->next_out;
  __pyx_v_zst.next_out = __pyx_t_2;

  /* "bgzip_utils.pyx":89
 *     zst.next_out = block.next_out
 * 
 *     err = inflateInit2(&zst, -15)             # <<<<<<<<<<<<<<
 *     if Z_OK != err:
 *         c_exception("Failed to initialize zlib compression")
 */
  __pyx_v_err = inflateInit2((&__pyx_v_zst), -15);

  /* "bgzip_utils.pyx":90
 * 
 *     err = inflateInit2(&zst, -15)
 *     if Z_OK != err:             # <<<<<<<<<<<<<<
 *         c_exception("Failed to initialize zlib compression")
 *     err = inflate(&zst, Z_FINISH)
 */
  __pyx_t_3 = ((Z_OK != __pyx_v_err) != 0);
  if (__pyx_t_3) {

    /* "bgzip_utils.pyx":91
 *     err = inflateInit2(&zst, -15)
 *     if Z_OK != err:
 *         c_exception("Failed to initialize zlib compression")             # <<<<<<<<<<<<<<
 *     err = inflate(&zst, Z_FINISH)
 *     if Z_STREAM_END == err:
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Failed to initialize zlib compression"));

    /* "bgzip_utils.pyx":90
 * 
 *     err = inflateInit2(&zst, -15)
 *     if Z_OK != err:             # <<<<<<<<<<<<<<
 *         c_exception("Failed to initialize zlib compression")
 *     err = inflate(&zst, Z_FINISH)
 */
  }

  /* "bgzip_utils.pyx":92
 *     if Z_OK != err:
 *         c_exception("Failed to initialize zlib compression")
 *     err = inflate(&zst, Z_FINISH)             # <<<<<<<<<<<<<<
 *     if Z_STREAM_END == err:
 *         pass
 */
  __pyx_v_err = inflate((&__pyx_v_zst), Z_FINISH);

  /* "bgzip_utils.pyx":93
 *         c_exception("Failed to initialize zlib compression")
 *     err = inflate(&zst, Z_FINISH)
 *     if Z_STREAM_END == err:             # <<<<<<<<<<<<<<
 *         pass
 *     else:
 */
  __pyx_t_3 = ((Z_STREAM_END == __pyx_v_err) != 0);
  if (__pyx_t_3) {
    goto __pyx_L4;
  }

  /* "bgzip_utils.pyx":96
 *         pass
 *     else:
 *         c_exception("Compression error encountered")             # <<<<<<<<<<<<<<
 *     inflateEnd(&zst)
 * 
 */
  /*else*/ {
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Compression error encountered"));
  }
  __pyx_L4:;

  /* "bgzip_utils.pyx":97
 *     else:
 *         c_exception("Compression error encountered")
 *     inflateEnd(&zst)             # <<<<<<<<<<<<<<
 * 
 *     if block[0].inflated_size != zst.total_out:
 */
  (void)(inflateEnd((&__pyx_v_zst)));

  /* "bgzip_utils.pyx":99
 *     inflateEnd(&zst)
 * 
 *     if block[0].inflated_size != zst.total_out:             # <<<<<<<<<<<<<<
 *         c_exception("Block differently sized than expected")
 * 
 */
  __pyx_t_3 = (((__pyx_v_block[0]).inflated_size != __pyx_v_zst.total_out) != 0);
  if (__pyx_t_3) {

    /* "bgzip_utils.pyx":100
 * 
 *     if block[0].inflated_size != zst.total_out:
 *         c_exception("Block differently sized than expected")             # <<<<<<<<<<<<<<
 * 
 *     if block.crc != crc32(0, block.next_out, block.inflated_size):
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Block differently sized than expected"));

    /* "bgzip_utils.pyx":99
 *     inflateEnd(&zst)
 * 
 *     if block[0].inflated_size != zst.total_out:             # <<<<<<<<<<<<<<
 *         c_exception("Block differently sized than expected")
 * 
 */
  }

  /* "bgzip_utils.pyx":102
 *         c_exception("Block differently sized than expected")
 * 
 *     if block.crc != crc32(0, block.next_out, block.inflated_size):             # <<<<<<<<<<<<<<
 *         c_exception("crc32 mismatch!")
 * 
 */
  __pyx_t_3 = ((__pyx_v_block->crc != crc32(0, __pyx_v_block->next_out, __pyx_v_block->inflated_size)) != 0);
  if (__pyx_t_3) {

    /* "bgzip_utils.pyx":103
 * 
 *     if block.crc != crc32(0, block.next_out, block.inflated_size):
 *         c_exception("crc32 mismatch!")             # <<<<<<<<<<<<<<
 * 
 *     # Difference betwwen `compress` and `deflate`:
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"crc32 mismatch!"));

    /* "bgzip_utils.pyx":102
 *         c_exception("Block differently sized than expected")
 * 
 *     if block.crc != crc32(0, block.next_out, block.inflated_size):             # <<<<<<<<<<<<<<
 *         c_exception("crc32 mismatch!")
 * 
 */
  }

  /* "bgzip_utils.pyx":77
 * 
 * 
 * cdef void inflate_block(Block * block) nogil:             # <<<<<<<<<<<<<<
 *     cdef z_stream zst
 *     cdef int err
 */

  /* function exit code */
}

/* "bgzip_utils.pyx":109
 * 
 * 
 * cdef void * ref_and_advance(BGZipStream * rb, unsigned int member_size, int *err) nogil:             # <<<<<<<<<<<<<<
 *     if rb.available_in  < member_size:
 *         err[0] = BGZIP_INSUFFICIENT_BYTES
 */

static void *__pyx_f_11bgzip_utils_ref_and_advance(__pyx_t_11bgzip_utils_BGZipStream *__pyx_v_rb, unsigned int __pyx_v_member_size, int *__pyx_v_err) {
  __pyx_t_5czlib_Bytef *__pyx_v_ret_val;
  void *__pyx_r;
  int __pyx_t_1;
  __pyx_t_5czlib_Bytef *__pyx_t_2;

  /* "bgzip_utils.pyx":110
 * 
 * cdef void * ref_and_advance(BGZipStream * rb, unsigned int member_size, int *err) nogil:
 *     if rb.available_in  < member_size:             # <<<<<<<<<<<<<<
 *         err[0] = BGZIP_INSUFFICIENT_BYTES
 *         return NULL
 */
  __pyx_t_1 = ((__pyx_v_rb->available_in < __pyx_v_member_size) != 0);
  if (__pyx_t_1) {

    /* "bgzip_utils.pyx":111
 * cdef void * ref_and_advance(BGZipStream * rb, unsigned int member_size, int *err) nogil:
 *     if rb.available_in  < member_size:
 *         err[0] = BGZIP_INSUFFICIENT_BYTES             # <<<<<<<<<<<<<<
 *         return NULL
 *     else:
 */
    (__pyx_v_err[0]) = __pyx_e_11bgzip_utils_BGZIP_INSUFFICIENT_BYTES;

    /* "bgzip_utils.pyx":112
 *     if rb.available_in  < member_size:
 *         err[0] = BGZIP_INSUFFICIENT_BYTES
 *         return NULL             # <<<<<<<<<<<<<<
 *     else:
 *         ret_val = rb.next_in
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "bgzip_utils.pyx":110
 * 
 * cdef void * ref_and_advance(BGZipStream * rb, unsigned int member_size, int *err) nogil:
 *     if rb.available_in  < member_size:             # <<<<<<<<<<<<<<
 *         err[0] = BGZIP_INSUFFICIENT_BYTES
 *         return NULL
 */
  }

  /* "bgzip_utils.pyx":114
 *         return NULL
 *     else:
 *         ret_val = rb.next_in             # <<<<<<<<<<<<<<
 *         rb.next_in += member_size
 *         rb.available_in  -= member_size
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_rb->next_in;
    __pyx_v_ret_val = __pyx_t_2;

    /* "bgzip_utils.pyx":115
 *     else:
 *         ret_val = rb.next_in
 *         rb.next_in += member_size             # <<<<<<<<<<<<<<
 *         rb.available_in  -= member_size
 *         err[0] = BGZIP_OK
 */
    __pyx_v_rb->next_in = (__pyx_v_rb->next_in + __pyx_v_member_size);

    /* "bgzip_utils.pyx":116
 *         ret_val = rb.next_in
 *         rb.next_in += member_size
 *         rb.available_in  -= member_size             # <<<<<<<<<<<<<<
 *         err[0] = BGZIP_OK
 *         return ret_val
 */
    __pyx_v_rb->available_in = (__pyx_v_rb->available_in - __pyx_v_member_size);

    /* "bgzip_utils.pyx":117
 *         rb.next_in += member_size
 *         rb.available_in  -= member_size
 *         err[0] = BGZIP_OK             # <<<<<<<<<<<<<<
 *         return ret_val
 * 
 */
    (__pyx_v_err[0]) = __pyx_e_11bgzip_utils_BGZIP_OK;

    /* "bgzip_utils.pyx":118
 *         rb.available_in  -= member_size
 *         err[0] = BGZIP_OK
 *         return ret_val             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_v_ret_val;
    goto __pyx_L0;
  }

  /* "bgzip_utils.pyx":109
 * 
 * 
 * cdef void * ref_and_advance(BGZipStream * rb, unsigned int member_size, int *err) nogil:             # <<<<<<<<<<<<<<
 *     if rb.available_in  < member_size:
 *         err[0] = BGZIP_INSUFFICIENT_BYTES
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "bgzip_utils.pyx":121
 * 
 * 
 * cdef int read_block(Block * block, BGZipStream *src) nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int i
 *     cdef int err
 */

static int __pyx_f_11bgzip_utils_read_block(__pyx_t_11bgzip_utils_Block *__pyx_v_block, __pyx_t_11bgzip_utils_BGZipStream *__pyx_v_src) {
  unsigned int __pyx_v_i;
  int __pyx_v_err;
  __pyx_t_11bgzip_utils_BlockHeader *__pyx_v_head;
  __pyx_t_11bgzip_utils_BlockTailer *__pyx_v_tail;
  __pyx_t_11bgzip_utils_BlockHeaderSubfield *__pyx_v_subfield;
  __pyx_t_5czlib_Bytef *__pyx_v_subfield_data;
  unsigned int __pyx_v_extra_len;
  int __pyx_r;
  int __pyx_t_1;
  unsigned int __pyx_t_2;
  unsigned int __pyx_t_3;
  unsigned int __pyx_t_4;
  unsigned short __pyx_t_5;
  int __pyx_t_6;
  __pyx_t_5czlib_Bytef *__pyx_t_7;

  /* "bgzip_utils.pyx":130
 *     cdef unsigned int extra_len
 * 
 *     block.block_size = 0             # <<<<<<<<<<<<<<
 * 
 *     head = <BlockHeader *>ref_and_advance(src, sizeof(BlockHeader), &err)
 */
  __pyx_v_block->block_size = 0;

  /* "bgzip_utils.pyx":132
 *     block.block_size = 0
 * 
 *     head = <BlockHeader *>ref_and_advance(src, sizeof(BlockHeader), &err)             # <<<<<<<<<<<<<<
 *     if err:
 *         return err
 */
  __pyx_v_head = ((__pyx_t_11bgzip_utils_BlockHeader *)__pyx_f_11bgzip_utils_ref_and_advance(__pyx_v_src, (sizeof(__pyx_t_11bgzip_utils_BlockHeader)), (&__pyx_v_err)));

  /* "bgzip_utils.pyx":133
 * 
 *     head = <BlockHeader *>ref_and_advance(src, sizeof(BlockHeader), &err)
 *     if err:             # <<<<<<<<<<<<<<
 *         return err
 * 
 */
  __pyx_t_1 = (__pyx_v_err != 0);
  if (__pyx_t_1) {

    /* "bgzip_utils.pyx":134
 *     head = <BlockHeader *>ref_and_advance(src, sizeof(BlockHeader), &err)
 *     if err:
 *         return err             # <<<<<<<<<<<<<<
 * 
 *     for i in range(<unsigned int>MAGIC_LENGTH):
 */
    __pyx_r = __pyx_v_err;
    goto __pyx_L0;

    /* "bgzip_utils.pyx":133
 * 
 *     head = <BlockHeader *>ref_and_advance(src, sizeof(BlockHeader), &err)
 *     if err:             # <<<<<<<<<<<<<<
 *         return err
 * 
 */
  }

  /* "bgzip_utils.pyx":136
 *         return err
 * 
 *     for i in range(<unsigned int>MAGIC_LENGTH):             # <<<<<<<<<<<<<<
 *         if head.magic[i] != MAGIC[i]:
 *             c_exception("Magic not found in gzip header")
 */
  __pyx_t_2 = ((unsigned int)__pyx_e_11bgzip_utils_MAGIC_LENGTH);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "bgzip_utils.pyx":137
 * 
 *     for i in range(<unsigned int>MAGIC_LENGTH):
 *         if head.magic[i] != MAGIC[i]:             # <<<<<<<<<<<<<<
 *             c_exception("Magic not found in gzip header")
 * 
 */
    __pyx_t_1 = (((__pyx_v_head->magic[__pyx_v_i]) != (__pyx_v_11bgzip_utils_MAGIC[__pyx_v_i])) != 0);
    if (__pyx_t_1) {

      /* "bgzip_utils.pyx":138
 *     for i in range(<unsigned int>MAGIC_LENGTH):
 *         if head.magic[i] != MAGIC[i]:
 *             c_exception("Magic not found in gzip header")             # <<<<<<<<<<<<<<
 * 
 *     extra_len = head.extra_len
 */
      __pyx_f_11bgzip_utils_c_exception(((char const *)"Magic not found in gzip header"));

      /* "bgzip_utils.pyx":137
 * 
 *     for i in range(<unsigned int>MAGIC_LENGTH):
 *         if head.magic[i] != MAGIC[i]:             # <<<<<<<<<<<<<<
 *             c_exception("Magic not found in gzip header")
 * 
 */
    }
  }

  /* "bgzip_utils.pyx":140
 *             c_exception("Magic not found in gzip header")
 * 
 *     extra_len = head.extra_len             # <<<<<<<<<<<<<<
 *     while extra_len > 0:
 *         subfield = <BlockHeaderSubfield *>ref_and_advance(src, sizeof(BlockHeaderSubfield), &err)
 */
  __pyx_t_5 = __pyx_v_head->extra_len;
  __pyx_v_extra_len = __pyx_t_5;

  /* "bgzip_utils.pyx":141
 * 
 *     extra_len = head.extra_len
 *     while extra_len > 0:             # <<<<<<<<<<<<<<
 *         subfield = <BlockHeaderSubfield *>ref_and_advance(src, sizeof(BlockHeaderSubfield), &err)
 *         if err:
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_extra_len > 0) != 0);
    if (!__pyx_t_1) break;

    /* "bgzip_utils.pyx":142
 *     extra_len = head.extra_len
 *     while extra_len > 0:
 *         subfield = <BlockHeaderSubfield *>ref_and_advance(src, sizeof(BlockHeaderSubfield), &err)             # <<<<<<<<<<<<<<
 *         if err:
 *             return err
 */
    __pyx_v_subfield = ((__pyx_t_11bgzip_utils_BlockHeaderSubfield *)__pyx_f_11bgzip_utils_ref_and_advance(__pyx_v_src, (sizeof(__pyx_t_11bgzip_utils_BlockHeaderSubfield)), (&__pyx_v_err)));

    /* "bgzip_utils.pyx":143
 *     while extra_len > 0:
 *         subfield = <BlockHeaderSubfield *>ref_and_advance(src, sizeof(BlockHeaderSubfield), &err)
 *         if err:             # <<<<<<<<<<<<<<
 *             return err
 *         extra_len -= sizeof(BlockHeaderSubfield)
 */
    __pyx_t_1 = (__pyx_v_err != 0);
    if (__pyx_t_1) {

      /* "bgzip_utils.pyx":144
 *         subfield = <BlockHeaderSubfield *>ref_and_advance(src, sizeof(BlockHeaderSubfield), &err)
 *         if err:
 *             return err             # <<<<<<<<<<<<<<
 *         extra_len -= sizeof(BlockHeaderSubfield)
 * 
 */
      __pyx_r = __pyx_v_err;
      goto __pyx_L0;

      /* "bgzip_utils.pyx":143
 *     while extra_len > 0:
 *         subfield = <BlockHeaderSubfield *>ref_and_advance(src, sizeof(BlockHeaderSubfield), &err)
 *         if err:             # <<<<<<<<<<<<<<
 *             return err
 *         extra_len -= sizeof(BlockHeaderSubfield)
 */
    }

    /* "bgzip_utils.pyx":145
 *         if err:
 *             return err
 *         extra_len -= sizeof(BlockHeaderSubfield)             # <<<<<<<<<<<<<<
 * 
 *         subfield_data = <Bytef *>ref_and_advance(src, subfield.length, &err)
 */
    __pyx_v_extra_len = (__pyx_v_extra_len - (sizeof(__pyx_t_11bgzip_utils_BlockHeaderSubfield)));

    /* "bgzip_utils.pyx":147
 *         extra_len -= sizeof(BlockHeaderSubfield)
 * 
 *         subfield_data = <Bytef *>ref_and_advance(src, subfield.length, &err)             # <<<<<<<<<<<<<<
 *         if err:
 *             return err
 */
    __pyx_v_subfield_data = ((__pyx_t_5czlib_Bytef *)__pyx_f_11bgzip_utils_ref_and_advance(__pyx_v_src, __pyx_v_subfield->length, (&__pyx_v_err)));

    /* "bgzip_utils.pyx":148
 * 
 *         subfield_data = <Bytef *>ref_and_advance(src, subfield.length, &err)
 *         if err:             # <<<<<<<<<<<<<<
 *             return err
 *         extra_len -= sizeof(subfield.length)
 */
    __pyx_t_1 = (__pyx_v_err != 0);
    if (__pyx_t_1) {

      /* "bgzip_utils.pyx":149
 *         subfield_data = <Bytef *>ref_and_advance(src, subfield.length, &err)
 *         if err:
 *             return err             # <<<<<<<<<<<<<<
 *         extra_len -= sizeof(subfield.length)
 * 
 */
      __pyx_r = __pyx_v_err;
      goto __pyx_L0;

      /* "bgzip_utils.pyx":148
 * 
 *         subfield_data = <Bytef *>ref_and_advance(src, subfield.length, &err)
 *         if err:             # <<<<<<<<<<<<<<
 *             return err
 *         extra_len -= sizeof(subfield.length)
 */
    }

    /* "bgzip_utils.pyx":150
 *         if err:
 *             return err
 *         extra_len -= sizeof(subfield.length)             # <<<<<<<<<<<<<<
 * 
 *         if b"B" == subfield.id_[0] and b"C" == subfield.id_[1]:
 */
    __pyx_v_extra_len = (__pyx_v_extra_len - (sizeof(__pyx_v_subfield->length)));

    /* "bgzip_utils.pyx":152
 *         extra_len -= sizeof(subfield.length)
 * 
 *         if b"B" == subfield.id_[0] and b"C" == subfield.id_[1]:             # <<<<<<<<<<<<<<
 *             if subfield.length != 2:
 *                 c_exception("Unexpected subfield length in gzip header")
 */
    __pyx_t_6 = (('B' == (__pyx_v_subfield->id_[0])) != 0);
    if (__pyx_t_6) {
    } else {
      __pyx_t_1 = __pyx_t_6;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_6 = (('C' == (__pyx_v_subfield->id_[1])) != 0);
    __pyx_t_1 = __pyx_t_6;
    __pyx_L12_bool_binop_done:;
    if (__pyx_t_1) {

      /* "bgzip_utils.pyx":153
 * 
 *         if b"B" == subfield.id_[0] and b"C" == subfield.id_[1]:
 *             if subfield.length != 2:             # <<<<<<<<<<<<<<
 *                 c_exception("Unexpected subfield length in gzip header")
 *             block.block_size = (<unsigned short *>subfield_data)[0]
 */
      __pyx_t_1 = ((__pyx_v_subfield->length != 2) != 0);
      if (__pyx_t_1) {

        /* "bgzip_utils.pyx":154
 *         if b"B" == subfield.id_[0] and b"C" == subfield.id_[1]:
 *             if subfield.length != 2:
 *                 c_exception("Unexpected subfield length in gzip header")             # <<<<<<<<<<<<<<
 *             block.block_size = (<unsigned short *>subfield_data)[0]
 * 
 */
        __pyx_f_11bgzip_utils_c_exception(((char const *)"Unexpected subfield length in gzip header"));

        /* "bgzip_utils.pyx":153
 * 
 *         if b"B" == subfield.id_[0] and b"C" == subfield.id_[1]:
 *             if subfield.length != 2:             # <<<<<<<<<<<<<<
 *                 c_exception("Unexpected subfield length in gzip header")
 *             block.block_size = (<unsigned short *>subfield_data)[0]
 */
      }

      /* "bgzip_utils.pyx":155
 *             if subfield.length != 2:
 *                 c_exception("Unexpected subfield length in gzip header")
 *             block.block_size = (<unsigned short *>subfield_data)[0]             # <<<<<<<<<<<<<<
 * 
 *     if 0 != extra_len:
 */
      __pyx_v_block->block_size = (((unsigned short *)__pyx_v_subfield_data)[0]);

      /* "bgzip_utils.pyx":152
 *         extra_len -= sizeof(subfield.length)
 * 
 *         if b"B" == subfield.id_[0] and b"C" == subfield.id_[1]:             # <<<<<<<<<<<<<<
 *             if subfield.length != 2:
 *                 c_exception("Unexpected subfield length in gzip header")
 */
    }
  }

  /* "bgzip_utils.pyx":157
 *             block.block_size = (<unsigned short *>subfield_data)[0]
 * 
 *     if 0 != extra_len:             # <<<<<<<<<<<<<<
 *         c_exception("Unexpected header length")
 * 
 */
  __pyx_t_1 = ((0 != __pyx_v_extra_len) != 0);
  if (__pyx_t_1) {

    /* "bgzip_utils.pyx":158
 * 
 *     if 0 != extra_len:
 *         c_exception("Unexpected header length")             # <<<<<<<<<<<<<<
 * 
 *     if 0 >= block.block_size:
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Unexpected header length"));

    /* "bgzip_utils.pyx":157
 *             block.block_size = (<unsigned short *>subfield_data)[0]
 * 
 *     if 0 != extra_len:             # <<<<<<<<<<<<<<
 *         c_exception("Unexpected header length")
 * 
 */
  }

  /* "bgzip_utils.pyx":160
 *         c_exception("Unexpected header length")
 * 
 *     if 0 >= block.block_size:             # <<<<<<<<<<<<<<
 *         c_exception("Negative or missing block size.")
 * 
 */
  __pyx_t_1 = ((0 >= __pyx_v_block->block_size) != 0);
  if (__pyx_t_1) {

    /* "bgzip_utils.pyx":161
 * 
 *     if 0 >= block.block_size:
 *         c_exception("Negative or missing block size.")             # <<<<<<<<<<<<<<
 * 
 *     block.next_in = src.next_in
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Negative or missing block size."));

    /* "bgzip_utils.pyx":160
 *         c_exception("Unexpected header length")
 * 
 *     if 0 >= block.block_size:             # <<<<<<<<<<<<<<
 *         c_exception("Negative or missing block size.")
 * 
 */
  }

  /* "bgzip_utils.pyx":163
 *         c_exception("Negative or missing block size.")
 * 
 *     block.next_in = src.next_in             # <<<<<<<<<<<<<<
 *     block.deflated_size = 1 + block.block_size - sizeof(BlockHeader) - head.extra_len - sizeof(BlockTailer)
 * 
 */
  __pyx_t_7 = __pyx_v_src->next_in;
  __pyx_v_block->next_in = __pyx_t_7;

  /* "bgzip_utils.pyx":164
 * 
 *     block.next_in = src.next_in
 *     block.deflated_size = 1 + block.block_size - sizeof(BlockHeader) - head.extra_len - sizeof(BlockTailer)             # <<<<<<<<<<<<<<
 * 
 *     ref_and_advance(src, block.deflated_size, &err)
 */
  __pyx_v_block->deflated_size = ((((1 + __pyx_v_block->block_size) - (sizeof(__pyx_t_11bgzip_utils_BlockHeader))) - __pyx_v_head->extra_len) - (sizeof(__pyx_t_11bgzip_utils_BlockTailer)));

  /* "bgzip_utils.pyx":166
 *     block.deflated_size = 1 + block.block_size - sizeof(BlockHeader) - head.extra_len - sizeof(BlockTailer)
 * 
 *     ref_and_advance(src, block.deflated_size, &err)             # <<<<<<<<<<<<<<
 *     if err:
 *         return err
 */
  (void)(__pyx_f_11bgzip_utils_ref_and_advance(__pyx_v_src, __pyx_v_block->deflated_size, (&__pyx_v_err)));

  /* "bgzip_utils.pyx":167
 * 
 *     ref_and_advance(src, block.deflated_size, &err)
 *     if err:             # <<<<<<<<<<<<<<
 *         return err
 * 
 */
  __pyx_t_1 = (__pyx_v_err != 0);
  if (__pyx_t_1) {

    /* "bgzip_utils.pyx":168
 *     ref_and_advance(src, block.deflated_size, &err)
 *     if err:
 *         return err             # <<<<<<<<<<<<<<
 * 
 *     tail = <BlockTailer *>ref_and_advance(src, sizeof(BlockTailer), &err)
 */
    __pyx_r = __pyx_v_err;
    goto __pyx_L0;

    /* "bgzip_utils.pyx":167
 * 
 *     ref_and_advance(src, block.deflated_size, &err)
 *     if err:             # <<<<<<<<<<<<<<
 *         return err
 * 
 */
  }

  /* "bgzip_utils.pyx":170
 *         return err
 * 
 *     tail = <BlockTailer *>ref_and_advance(src, sizeof(BlockTailer), &err)             # <<<<<<<<<<<<<<
 *     if err:
 *         return err
 */
  __pyx_v_tail = ((__pyx_t_11bgzip_utils_BlockTailer *)__pyx_f_11bgzip_utils_ref_and_advance(__pyx_v_src, (sizeof(__pyx_t_11bgzip_utils_BlockTailer)), (&__pyx_v_err)));

  /* "bgzip_utils.pyx":171
 * 
 *     tail = <BlockTailer *>ref_and_advance(src, sizeof(BlockTailer), &err)
 *     if err:             # <<<<<<<<<<<<<<
 *         return err
 * 
 */
  __pyx_t_1 = (__pyx_v_err != 0);
  if (__pyx_t_1) {

    /* "bgzip_utils.pyx":172
 *     tail = <BlockTailer *>ref_and_advance(src, sizeof(BlockTailer), &err)
 *     if err:
 *         return err             # <<<<<<<<<<<<<<
 * 
 *     block.crc = tail.crc
 */
    __pyx_r = __pyx_v_err;
    goto __pyx_L0;

    /* "bgzip_utils.pyx":171
 * 
 *     tail = <BlockTailer *>ref_and_advance(src, sizeof(BlockTailer), &err)
 *     if err:             # <<<<<<<<<<<<<<
 *         return err
 * 
 */
  }

  /* "bgzip_utils.pyx":174
 *         return err
 * 
 *     block.crc = tail.crc             # <<<<<<<<<<<<<<
 *     block.inflated_size = tail.inflated_size
 * 
 */
  __pyx_t_2 = __pyx_v_tail->crc;
  __pyx_v_block->crc = __pyx_t_2;

  /* "bgzip_utils.pyx":175
 * 
 *     block.crc = tail.crc
 *     block.inflated_size = tail.inflated_size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_v_tail->inflated_size;
  __pyx_v_block->inflated_size = __pyx_t_2;

  /* "bgzip_utils.pyx":121
 * 
 * 
 * cdef int read_block(Block * block, BGZipStream *src) nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int i
 *     cdef int err
 */

  /* function exit code */
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "bgzip_utils.pyx":178
 * 
 * 
 * def decompress_into(bytes src_buff, bytearray dst_buff, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`, resizing `dst_buff` as needed.
 */

/* Python wrapper */
static PyObject *__pyx_pw_11bgzip_utils_1decompress_into(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11bgzip_utils_decompress_into[] = "\n    Inflate bytes from `src_buff` into `dst_buff`, resizing `dst_buff` as needed.\n    ";
static PyMethodDef __pyx_mdef_11bgzip_utils_1decompress_into = {"decompress_into", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_11bgzip_utils_1decompress_into, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11bgzip_utils_decompress_into};
static PyObject *__pyx_pw_11bgzip_utils_1decompress_into(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_src_buff = 0;
  PyObject *__pyx_v_dst_buff = 0;
  CYTHON_UNUSED int __pyx_v_num_threads;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decompress_into (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_src_buff,&__pyx_n_s_dst_buff,&__pyx_n_s_num_threads,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_src_buff)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dst_buff)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("decompress_into", 1, 3, 3, 1); __PYX_ERR(0, 178, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_num_threads)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("decompress_into", 1, 3, 3, 2); __PYX_ERR(0, 178, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "decompress_into") < 0)) __PYX_ERR(0, 178, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_src_buff = ((PyObject*)values[0]);
    __pyx_v_dst_buff = ((PyObject*)values[1]);
    __pyx_v_num_threads = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_num_threads == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 178, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("decompress_into", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 178, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bgzip_utils.decompress_into", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src_buff), (&PyBytes_Type), 1, "src_buff", 1))) __PYX_ERR(0, 178, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dst_buff), (&PyByteArray_Type), 1, "dst_buff", 1))) __PYX_ERR(0, 178, __pyx_L1_error)
  __pyx_r = __pyx_pf_11bgzip_utils_decompress_into(__pyx_self, __pyx_v_src_buff, __pyx_v_dst_buff, __pyx_v_num_threads);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11bgzip_utils_decompress_into(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_src_buff, PyObject *__pyx_v_dst_buff, CYTHON_UNUSED int __pyx_v_num_threads) {
  int __pyx_v_i;
  PyObject *__pyx_v_dst;
  __pyx_t_5czlib_Bytef *__pyx_v_out;
  unsigned int __pyx_v_offset;
  int __pyx_v_bytes_read;
  int __pyx_v_inflated_size;
  int __pyx_v_number_of_blocks;
  __pyx_t_11bgzip_utils_Block __pyx_v_blocks[__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS];
  __pyx_t_11bgzip_utils_BGZipStream __pyx_v_src;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  __pyx_t_5czlib_Bytef *__pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("decompress_into", 0);

  /* "bgzip_utils.pyx":183
 *     """
 *     cdef int i
 *     cdef PyObject * dst = <PyObject *>dst_buff             # <<<<<<<<<<<<<<
 *     cdef Bytef * out = NULL
 *     cdef unsigned int offset = len(dst_buff)
 */
  __pyx_v_dst = ((PyObject *)__pyx_v_dst_buff);

  /* "bgzip_utils.pyx":184
 *     cdef int i
 *     cdef PyObject * dst = <PyObject *>dst_buff
 *     cdef Bytef * out = NULL             # <<<<<<<<<<<<<<
 *     cdef unsigned int offset = len(dst_buff)
 *     cdef int bytes_read = 0, inflated_size = 0
 */
  __pyx_v_out = NULL;

  /* "bgzip_utils.pyx":185
 *     cdef PyObject * dst = <PyObject *>dst_buff
 *     cdef Bytef * out = NULL
 *     cdef unsigned int offset = len(dst_buff)             # <<<<<<<<<<<<<<
 *     cdef int bytes_read = 0, inflated_size = 0
 *     cdef int number_of_blocks = 0
 */
  if (unlikely(__pyx_v_dst_buff == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 185, __pyx_L1_error)
  }
  __pyx_t_1 = PyByteArray_GET_SIZE(__pyx_v_dst_buff); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 185, __pyx_L1_error)
  __pyx_v_offset = __pyx_t_1;

  /* "bgzip_utils.pyx":186
 *     cdef Bytef * out = NULL
 *     cdef unsigned int offset = len(dst_buff)
 *     cdef int bytes_read = 0, inflated_size = 0             # <<<<<<<<<<<<<<
 *     cdef int number_of_blocks = 0
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 */
  __pyx_v_bytes_read = 0;
  __pyx_v_inflated_size = 0;

  /* "bgzip_utils.pyx":187
 *     cdef unsigned int offset = len(dst_buff)
 *     cdef int bytes_read = 0, inflated_size = 0
 *     cdef int number_of_blocks = 0             # <<<<<<<<<<<<<<
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 */
  __pyx_v_number_of_blocks = 0;

  /* "bgzip_utils.pyx":191
 * 
 *     cdef BGZipStream src
 *     src.next_in = src_buff             # <<<<<<<<<<<<<<
 *     src.available_in  = len(src_buff)
 * 
 */
  if (unlikely(__pyx_v_src_buff == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 191, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyBytes_AsWritableUString(__pyx_v_src_buff); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 191, __pyx_L1_error)
  __pyx_v_src.next_in = __pyx_t_2;

  /* "bgzip_utils.pyx":192
 *     cdef BGZipStream src
 *     src.next_in = src_buff
 *     src.available_in  = len(src_buff)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  if (unlikely(__pyx_v_src_buff == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 192, __pyx_L1_error)
  }
  __pyx_t_1 = PyBytes_GET_SIZE(__pyx_v_src_buff); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 192, __pyx_L1_error)
  __pyx_v_src.available_in = __pyx_t_1;

  /* "bgzip_utils.pyx":194
 *     src.available_in  = len(src_buff)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "bgzip_utils.pyx":195
 * 
 *     with nogil:
 *         for i in range(NUMBER_OF_BLOCKS):             # <<<<<<<<<<<<<<
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break
 */
        __pyx_t_3 = __pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS;
        __pyx_t_4 = __pyx_t_3;
        for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
          __pyx_v_i = __pyx_t_5;

          /* "bgzip_utils.pyx":196
 *     with nogil:
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):             # <<<<<<<<<<<<<<
 *                 break
 *             bytes_read += 1 + blocks[i].block_size
 */
          __pyx_t_6 = ((__pyx_e_11bgzip_utils_BGZIP_INSUFFICIENT_BYTES == __pyx_f_11bgzip_utils_read_block((&(__pyx_v_blocks[__pyx_v_i])), (&__pyx_v_src))) != 0);
          if (__pyx_t_6) {

            /* "bgzip_utils.pyx":197
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break             # <<<<<<<<<<<<<<
 *             bytes_read += 1 + blocks[i].block_size
 *             inflated_size += blocks[i].inflated_size
 */
            goto __pyx_L7_break;

            /* "bgzip_utils.pyx":196
 *     with nogil:
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):             # <<<<<<<<<<<<<<
 *                 break
 *             bytes_read += 1 + blocks[i].block_size
 */
          }

          /* "bgzip_utils.pyx":198
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break
 *             bytes_read += 1 + blocks[i].block_size             # <<<<<<<<<<<<<<
 *             inflated_size += blocks[i].inflated_size
 *             number_of_blocks += 1
 */
          __pyx_v_bytes_read = (__pyx_v_bytes_read + (1 + (__pyx_v_blocks[__pyx_v_i]).block_size));

          /* "bgzip_utils.pyx":199
 *                 break
 *             bytes_read += 1 + blocks[i].block_size
 *             inflated_size += blocks[i].inflated_size             # <<<<<<<<<<<<<<
 *             number_of_blocks += 1
 * 
 */
          __pyx_v_inflated_size = (__pyx_v_inflated_size + (__pyx_v_blocks[__pyx_v_i]).inflated_size);

          /* "bgzip_utils.pyx":200
 *             bytes_read += 1 + blocks[i].block_size
 *             inflated_size += blocks[i].inflated_size
 *             number_of_blocks += 1             # <<<<<<<<<<<<<<
 * 
 *     PyByteArray_Resize(dst, offset + inflated_size)
 */
          __pyx_v_number_of_blocks = (__pyx_v_number_of_blocks + 1);
        }
        __pyx_L7_break:;
      }

      /* "bgzip_utils.pyx":194
 *     src.available_in  = len(src_buff)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "bgzip_utils.pyx":202
 *             number_of_blocks += 1
 * 
 *     PyByteArray_Resize(dst, offset + inflated_size)             # <<<<<<<<<<<<<<
 *     out = <Bytef *>PyByteArray_AS_STRING(dst)
 *     out += offset
 */
  (void)(PyByteArray_Resize(__pyx_v_dst, (__pyx_v_offset + __pyx_v_inflated_size)));

  /* "bgzip_utils.pyx":203
 * 
 *     PyByteArray_Resize(dst, offset + inflated_size)
 *     out = <Bytef *>PyByteArray_AS_STRING(dst)             # <<<<<<<<<<<<<<
 *     out += offset
 * 
 */
  __pyx_v_out = ((__pyx_t_5czlib_Bytef *)PyByteArray_AS_STRING(__pyx_v_dst));

  /* "bgzip_utils.pyx":204
 *     PyByteArray_Resize(dst, offset + inflated_size)
 *     out = <Bytef *>PyByteArray_AS_STRING(dst)
 *     out += offset             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_v_out = (__pyx_v_out + __pyx_v_offset);

  /* "bgzip_utils.pyx":206
 *     out += offset
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(number_of_blocks):
 *             blocks[i].next_out = out
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "bgzip_utils.pyx":207
 * 
 *     with nogil:
 *         for i in range(number_of_blocks):             # <<<<<<<<<<<<<<
 *             blocks[i].next_out = out
 *             out += blocks[i].inflated_size
 */
        __pyx_t_5 = __pyx_v_number_of_blocks;
        __pyx_t_7 = __pyx_t_5;
        for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
          __pyx_v_i = __pyx_t_8;

          /* "bgzip_utils.pyx":208
 *     with nogil:
 *         for i in range(number_of_blocks):
 *             blocks[i].next_out = out             # <<<<<<<<<<<<<<
 *             out += blocks[i].inflated_size
 * 
 */
          (__pyx_v_blocks[__pyx_v_i]).next_out = __pyx_v_out;

          /* "bgzip_utils.pyx":209
 *         for i in range(number_of_blocks):
 *             blocks[i].next_out = out
 *             out += blocks[i].inflated_size             # <<<<<<<<<<<<<<
 * 
 *         for i in prange(number_of_blocks, num_threads=num_threads, schedule="dynamic"):
 */
          __pyx_v_out = (__pyx_v_out + (__pyx_v_blocks[__pyx_v_i]).inflated_size);
        }

        /* "bgzip_utils.pyx":211
 *             out += blocks[i].inflated_size
 * 
 *         for i in prange(number_of_blocks, num_threads=num_threads, schedule="dynamic"):             # <<<<<<<<<<<<<<
 *             inflate_block(&blocks[i])
 * 
 */
        __pyx_t_5 = __pyx_v_number_of_blocks;
        if (1 == 0) abort();
        {
            #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
                #undef likely
                #undef unlikely
                #define likely(x)   (x)
                #define unlikely(x) (x)
            #endif
            __pyx_t_8 = (__pyx_t_5 - 0 + 1 - 1/abs(1)) / 1;
            if (__pyx_t_8 > 0)
            {
                #ifdef _OPENMP
                #pragma omp parallel num_threads(__pyx_v_num_threads)
                #endif /* _OPENMP */
                {
                    #ifdef _OPENMP
                    #pragma omp for firstprivate(__pyx_v_i) lastprivate(__pyx_v_i) schedule(dynamic)
                    #endif /* _OPENMP */
                    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_8; __pyx_t_7++){
                        {
                            __pyx_v_i = (int)(0 + 1 * __pyx_t_7);

                            /* "bgzip_utils.pyx":212
 * 
 *         for i in prange(number_of_blocks, num_threads=num_threads, schedule="dynamic"):
 *             inflate_block(&blocks[i])             # <<<<<<<<<<<<<<
 * 
 *     return bytes_read
 */
                            __pyx_f_11bgzip_utils_inflate_block((&(__pyx_v_blocks[__pyx_v_i])));
                        }
                    }
                }
            }
        }
        #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
            #undef likely
            #undef unlikely
            #define likely(x)   __builtin_expect(!!(x), 1)
            #define unlikely(x) __builtin_expect(!!(x), 0)
        #endif
      }

      /* "bgzip_utils.pyx":206
 *     out += offset
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(number_of_blocks):
 *             blocks[i].next_out = out
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L11;
        }
        __pyx_L11:;
      }
  }

  /* "bgzip_utils.pyx":214
 *             inflate_block(&blocks[i])
 * 
 *     return bytes_read             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_bytes_read); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "bgzip_utils.pyx":178
 * 
 * 
 * def decompress_into(bytes src_buff, bytearray dst_buff, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`, resizing `dst_buff` as needed.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("bgzip_utils.decompress_into", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bgzip_utils.pyx":217
 * 
 * 
 * def decompress_into_2(bytes src_buff, object dst_buff_obj, unsigned int offset, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`
 */

/* Python wrapper */
static PyObject *__pyx_pw_11bgzip_utils_3decompress_into_2(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11bgzip_utils_2decompress_into_2[] = "\n    Inflate bytes from `src_buff` into `dst_buff`\n    ";
static PyMethodDef __pyx_mdef_11bgzip_utils_3decompress_into_2 = {"decompress_into_2", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_11bgzip_utils_3decompress_into_2, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11bgzip_utils_2decompress_into_2};
static PyObject *__pyx_pw_11bgzip_utils_3decompress_into_2(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_src_buff = 0;
  PyObject *__pyx_v_dst_buff_obj = 0;
  unsigned int __pyx_v_offset;
  CYTHON_UNUSED int __pyx_v_num_threads;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decompress_into_2 (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_src_buff,&__pyx_n_s_dst_buff_obj,&__pyx_n_s_offset,&__pyx_n_s_num_threads,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_src_buff)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dst_buff_obj)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("decompress_into_2", 1, 4, 4, 1); __PYX_ERR(0, 217, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_offset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("decompress_into_2", 1, 4, 4, 2); __PYX_ERR(0, 217, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_num_threads)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("decompress_into_2", 1, 4, 4, 3); __PYX_ERR(0, 217, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "decompress_into_2") < 0)) __PYX_ERR(0, 217, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_src_buff = ((PyObject*)values[0]);
    __pyx_v_dst_buff_obj = values[1];
    __pyx_v_offset = __Pyx_PyInt_As_unsigned_int(values[2]); if (unlikely((__pyx_v_offset == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 217, __pyx_L3_error)
    __pyx_v_num_threads = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_num_threads == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 217, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("decompress_into_2", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 217, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bgzip_utils.decompress_into_2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src_buff), (&PyBytes_Type), 1, "src_buff", 1))) __PYX_ERR(0, 217, __pyx_L1_error)
  __pyx_r = __pyx_pf_11bgzip_utils_2decompress_into_2(__pyx_self, __pyx_v_src_buff, __pyx_v_dst_buff_obj, __pyx_v_offset, __pyx_v_num_threads);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11bgzip_utils_2decompress_into_2(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_src_buff, PyObject *__pyx_v_dst_buff_obj, unsigned int __pyx_v_offset, CYTHON_UNUSED int __pyx_v_num_threads) {
  int __pyx_v_i;
  __pyx_t_5czlib_Bytef *__pyx_v_out;
  unsigned int __pyx_v_bytes_read;
  unsigned int __pyx_v_bytes_inflated;
  int __pyx_v_number_of_blocks;
  __pyx_t_11bgzip_utils_Block __pyx_v_blocks[__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS];
  __pyx_t_11bgzip_utils_BGZipStream __pyx_v_src;
  PyObject *__pyx_v_dst_buff;
  unsigned int __pyx_v_avail_out;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_5czlib_Bytef *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("decompress_into_2", 0);

  /* "bgzip_utils.pyx":222
 *     """
 *     cdef int i
 *     cdef Bytef * out = NULL             # <<<<<<<<<<<<<<
 *     cdef unsigned int bytes_read = 0, bytes_inflated = 0
 *     cdef int number_of_blocks = 0
 */
  __pyx_v_out = NULL;

  /* "bgzip_utils.pyx":223
 *     cdef int i
 *     cdef Bytef * out = NULL
 *     cdef unsigned int bytes_read = 0, bytes_inflated = 0             # <<<<<<<<<<<<<<
 *     cdef int number_of_blocks = 0
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 */
  __pyx_v_bytes_read = 0;
  __pyx_v_bytes_inflated = 0;

  /* "bgzip_utils.pyx":224
 *     cdef Bytef * out = NULL
 *     cdef unsigned int bytes_read = 0, bytes_inflated = 0
 *     cdef int number_of_blocks = 0             # <<<<<<<<<<<<<<
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 */
  __pyx_v_number_of_blocks = 0;

  /* "bgzip_utils.pyx":228
 * 
 *     cdef BGZipStream src
 *     src.next_in = src_buff             # <<<<<<<<<<<<<<
 *     src.available_in = len(src_buff)
 * 
 */
  if (unlikely(__pyx_v_src_buff == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 228, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyBytes_AsWritableUString(__pyx_v_src_buff); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 228, __pyx_L1_error)
  __pyx_v_src.next_in = __pyx_t_1;

  /* "bgzip_utils.pyx":229
 *     cdef BGZipStream src
 *     src.next_in = src_buff
 *     src.available_in = len(src_buff)             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject * dst_buff = <PyObject *>dst_buff_obj
 */
  if (unlikely(__pyx_v_src_buff == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 229, __pyx_L1_error)
  }
  __pyx_t_2 = PyBytes_GET_SIZE(__pyx_v_src_buff); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 229, __pyx_L1_error)
  __pyx_v_src.available_in = __pyx_t_2;

  /* "bgzip_utils.pyx":231
 *     src.available_in = len(src_buff)
 * 
 *     cdef PyObject * dst_buff = <PyObject *>dst_buff_obj             # <<<<<<<<<<<<<<
 *     if PyMemoryView_Check(dst_buff):
 *         # TODO: Check buffer is contiguous, has normal stride
 */
  __pyx_v_dst_buff = ((PyObject *)__pyx_v_dst_buff_obj);

  /* "bgzip_utils.pyx":232
 * 
 *     cdef PyObject * dst_buff = <PyObject *>dst_buff_obj
 *     if PyMemoryView_Check(dst_buff):             # <<<<<<<<<<<<<<
 *         # TODO: Check buffer is contiguous, has normal stride
 *         out  = <Bytef *>(<Py_buffer *>PyMemoryView_GET_BUFFER(dst_buff)).buf
 */
  __pyx_t_3 = (PyMemoryView_Check(__pyx_v_dst_buff) != 0);
  if (likely(__pyx_t_3)) {

    /* "bgzip_utils.pyx":234
 *     if PyMemoryView_Check(dst_buff):
 *         # TODO: Check buffer is contiguous, has normal stride
 *         out  = <Bytef *>(<Py_buffer *>PyMemoryView_GET_BUFFER(dst_buff)).buf             # <<<<<<<<<<<<<<
 *         assert NULL != out
 *     else:
 */
    __pyx_v_out = ((__pyx_t_5czlib_Bytef *)((Py_buffer *)PyMemoryView_GET_BUFFER(__pyx_v_dst_buff))->buf);

    /* "bgzip_utils.pyx":235
 *         # TODO: Check buffer is contiguous, has normal stride
 *         out  = <Bytef *>(<Py_buffer *>PyMemoryView_GET_BUFFER(dst_buff)).buf
 *         assert NULL != out             # <<<<<<<<<<<<<<
 *     else:
 *         raise Exception("dst_buff_obj must be a memoryview instance.")
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!((NULL != __pyx_v_out) != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 235, __pyx_L1_error)
      }
    }
    #endif

    /* "bgzip_utils.pyx":232
 * 
 *     cdef PyObject * dst_buff = <PyObject *>dst_buff_obj
 *     if PyMemoryView_Check(dst_buff):             # <<<<<<<<<<<<<<
 *         # TODO: Check buffer is contiguous, has normal stride
 *         out  = <Bytef *>(<Py_buffer *>PyMemoryView_GET_BUFFER(dst_buff)).buf
 */
    goto __pyx_L3;
  }

  /* "bgzip_utils.pyx":237
 *         assert NULL != out
 *     else:
 *         raise Exception("dst_buff_obj must be a memoryview instance.")             # <<<<<<<<<<<<<<
 *         # TODO: support bytearray objects
 * 
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_tuple_, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 237, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "bgzip_utils.pyx":240
 *         # TODO: support bytearray objects
 * 
 *     cdef unsigned int avail_out = PySequence_Size(dst_buff) - offset             # <<<<<<<<<<<<<<
 *     out += offset
 * 
 */
  __pyx_v_avail_out = (PySequence_Size(__pyx_v_dst_buff) - __pyx_v_offset);

  /* "bgzip_utils.pyx":241
 * 
 *     cdef unsigned int avail_out = PySequence_Size(dst_buff) - offset
 *     out += offset             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_v_out = (__pyx_v_out + __pyx_v_offset);

  /* "bgzip_utils.pyx":243
 *     out += offset
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "bgzip_utils.pyx":244
 * 
 *     with nogil:
 *         for i in range(NUMBER_OF_BLOCKS):             # <<<<<<<<<<<<<<
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break
 */
        __pyx_t_5 = __pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS;
        __pyx_t_6 = __pyx_t_5;
        for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
          __pyx_v_i = __pyx_t_7;

          /* "bgzip_utils.pyx":245
 *     with nogil:
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):             # <<<<<<<<<<<<<<
 *                 break
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:
 */
          __pyx_t_3 = ((__pyx_e_11bgzip_utils_BGZIP_INSUFFICIENT_BYTES == __pyx_f_11bgzip_utils_read_block((&(__pyx_v_blocks[__pyx_v_i])), (&__pyx_v_src))) != 0);
          if (__pyx_t_3) {

            /* "bgzip_utils.pyx":246
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break             # <<<<<<<<<<<<<<
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:
 *                 break
 */
            goto __pyx_L8_break;

            /* "bgzip_utils.pyx":245
 *     with nogil:
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):             # <<<<<<<<<<<<<<
 *                 break
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:
 */
          }

          /* "bgzip_utils.pyx":247
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:             # <<<<<<<<<<<<<<
 *                 break
 *             bytes_read += 1 + blocks[i].block_size
 */
          __pyx_t_3 = ((__pyx_v_avail_out < (__pyx_v_bytes_inflated + (__pyx_v_blocks[__pyx_v_i]).inflated_size)) != 0);
          if (__pyx_t_3) {

            /* "bgzip_utils.pyx":248
 *                 break
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:
 *                 break             # <<<<<<<<<<<<<<
 *             bytes_read += 1 + blocks[i].block_size
 *             bytes_inflated += blocks[i].inflated_size
 */
            goto __pyx_L8_break;

            /* "bgzip_utils.pyx":247
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 *                 break
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:             # <<<<<<<<<<<<<<
 *                 break
 *             bytes_read += 1 + blocks[i].block_size
 */
          }

          /* "bgzip_utils.pyx":249
 *             if avail_out < bytes_inflated + blocks[i].inflated_size:
 *                 break
 *             bytes_read += 1 + blocks[i].block_size             # <<<<<<<<<<<<<<
 *             bytes_inflated += blocks[i].inflated_size
 *             number_of_blocks += 1
 */
          __pyx_v_bytes_read = (__pyx_v_bytes_read + (1 + (__pyx_v_blocks[__pyx_v_i]).block_size));

          /* "bgzip_utils.pyx":250
 *                 break
 *             bytes_read += 1 + blocks[i].block_size
 *             bytes_inflated += blocks[i].inflated_size             # <<<<<<<<<<<<<<
 *             number_of_blocks += 1
 * 
 */
          __pyx_v_bytes_inflated = (__pyx_v_bytes_inflated + (__pyx_v_blocks[__pyx_v_i]).inflated_size);

          /* "bgzip_utils.pyx":251
 *             bytes_read += 1 + blocks[i].block_size
 *             bytes_inflated += blocks[i].inflated_size
 *             number_of_blocks += 1             # <<<<<<<<<<<<<<
 * 
 *         for i in range(number_of_blocks):
 */
          __pyx_v_number_of_blocks = (__pyx_v_number_of_blocks + 1);
        }
        __pyx_L8_break:;

        /* "bgzip_utils.pyx":253
 *             number_of_blocks += 1
 * 
 *         for i in range(number_of_blocks):             # <<<<<<<<<<<<<<
 *             blocks[i].next_out = out
 *             out += blocks[i].inflated_size
 */
        __pyx_t_7 = __pyx_v_number_of_blocks;
        __pyx_t_8 = __pyx_t_7;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "bgzip_utils.pyx":254
 * 
 *         for i in range(number_of_blocks):
 *             blocks[i].next_out = out             # <<<<<<<<<<<<<<
 *             out += blocks[i].inflated_size
 * 
 */
          (__pyx_v_blocks[__pyx_v_i]).next_out = __pyx_v_out;

          /* "bgzip_utils.pyx":255
 *         for i in range(number_of_blocks):
 *             blocks[i].next_out = out
 *             out += blocks[i].inflated_size             # <<<<<<<<<<<<<<
 * 
 *         for i in prange(number_of_blocks, num_threads=num_threads, schedule="dynamic"):
 */
          __pyx_v_out = (__pyx_v_out + (__pyx_v_blocks[__pyx_v_i]).inflated_size);
        }

        /* "bgzip_utils.pyx":257
 *             out += blocks[i].inflated_size
 * 
 *         for i in prange(number_of_blocks, num_threads=num_threads, schedule="dynamic"):             # <<<<<<<<<<<<<<
 *             inflate_block(&blocks[i])
 * 
 */
        __pyx_t_7 = __pyx_v_number_of_blocks;
        if (1 == 0) abort();
        {
            #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
                #undef likely
                #undef unlikely
                #define likely(x)   (x)
                #define unlikely(x) (x)
            #endif
            __pyx_t_9 = (__pyx_t_7 - 0 + 1 - 1/abs(1)) / 1;
            if (__pyx_t_9 > 0)
            {
                #ifdef _OPENMP
                #pragma omp parallel num_threads(__pyx_v_num_threads)
                #endif /* _OPENMP */
                {
                    #ifdef _OPENMP
                    #pragma omp for firstprivate(__pyx_v_i) lastprivate(__pyx_v_i) schedule(dynamic)
                    #endif /* _OPENMP */
                    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_9; __pyx_t_8++){
                        {
                            __pyx_v_i = (int)(0 + 1 * __pyx_t_8);

                            /* "bgzip_utils.pyx":258
 * 
 *         for i in prange(number_of_blocks, num_threads=num_threads, schedule="dynamic"):
 *             inflate_block(&blocks[i])             # <<<<<<<<<<<<<<
 * 
 *     return bytes_read, bytes_inflated
 */
                            __pyx_f_11bgzip_utils_inflate_block((&(__pyx_v_blocks[__pyx_v_i])));
                        }
                    }
                }
            }
        }
        #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
            #undef likely
            #undef unlikely
            #define likely(x)   __builtin_expect(!!(x), 1)
            #define unlikely(x) __builtin_expect(!!(x), 0)
        #endif
      }

      /* "bgzip_utils.pyx":243
 *     out += offset
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(NUMBER_OF_BLOCKS):
 *             if BGZIP_INSUFFICIENT_BYTES == read_block(&blocks[i], &src):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "bgzip_utils.pyx":260
 *             inflate_block(&blocks[i])
 * 
 *     return bytes_read, bytes_inflated             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_bytes_read); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = __Pyx_PyInt_From_unsigned_int(__pyx_v_bytes_inflated); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_10);
  __pyx_t_4 = 0;
  __pyx_t_10 = 0;
  __pyx_r = __pyx_t_11;
  __pyx_t_11 = 0;
  goto __pyx_L0;

  /* "bgzip_utils.pyx":217
 * 
 * 
 * def decompress_into_2(bytes src_buff, object dst_buff_obj, unsigned int offset, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("bgzip_utils.decompress_into_2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bgzip_utils.pyx":263
 * 
 * 
 * cdef void compress_block(Block * block) nogil:             # <<<<<<<<<<<<<<
 *     cdef z_stream zst
 *     cdef int err = 0
 */

static void __pyx_f_11bgzip_utils_compress_block(__pyx_t_11bgzip_utils_Block *__pyx_v_block) {
  __pyx_t_5czlib_z_stream __pyx_v_zst;
  int __pyx_v_err;
  __pyx_t_11bgzip_utils_BlockHeader *__pyx_v_head;
  __pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield *__pyx_v_head_subfield;
  __pyx_t_11bgzip_utils_BlockTailer *__pyx_v_tail;
  int __pyx_v_wbits;
  int __pyx_v_mem_level;
  int __pyx_v_i;
  __pyx_t_5czlib_Bytef *__pyx_t_1;
  unsigned int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  long __pyx_t_6;

  /* "bgzip_utils.pyx":265
 * cdef void compress_block(Block * block) nogil:
 *     cdef z_stream zst
 *     cdef int err = 0             # <<<<<<<<<<<<<<
 *     cdef BlockHeader * head
 *     cdef BlockHeaderBGZipSubfield * head_subfield
 */
  __pyx_v_err = 0;

  /* "bgzip_utils.pyx":269
 *     cdef BlockHeaderBGZipSubfield * head_subfield
 *     cdef BlockTailer * tail
 *     cdef int wbits = -15             # <<<<<<<<<<<<<<
 *     cdef int mem_level = 8
 * 
 */
  __pyx_v_wbits = -15;

  /* "bgzip_utils.pyx":270
 *     cdef BlockTailer * tail
 *     cdef int wbits = -15
 *     cdef int mem_level = 8             # <<<<<<<<<<<<<<
 * 
 *     head = <BlockHeader *>block.next_out
 */
  __pyx_v_mem_level = 8;

  /* "bgzip_utils.pyx":272
 *     cdef int mem_level = 8
 * 
 *     head = <BlockHeader *>block.next_out             # <<<<<<<<<<<<<<
 *     block.next_out += sizeof(BlockHeader)
 * 
 */
  __pyx_v_head = ((__pyx_t_11bgzip_utils_BlockHeader *)__pyx_v_block->next_out);

  /* "bgzip_utils.pyx":273
 * 
 *     head = <BlockHeader *>block.next_out
 *     block.next_out += sizeof(BlockHeader)             # <<<<<<<<<<<<<<
 * 
 *     head_subfield = <BlockHeaderBGZipSubfield *>block.next_out
 */
  __pyx_v_block->next_out = (__pyx_v_block->next_out + (sizeof(__pyx_t_11bgzip_utils_BlockHeader)));

  /* "bgzip_utils.pyx":275
 *     block.next_out += sizeof(BlockHeader)
 * 
 *     head_subfield = <BlockHeaderBGZipSubfield *>block.next_out             # <<<<<<<<<<<<<<
 *     block.next_out += sizeof(BlockHeaderBGZipSubfield)
 * 
 */
  __pyx_v_head_subfield = ((__pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield *)__pyx_v_block->next_out);

  /* "bgzip_utils.pyx":276
 * 
 *     head_subfield = <BlockHeaderBGZipSubfield *>block.next_out
 *     block.next_out += sizeof(BlockHeaderBGZipSubfield)             # <<<<<<<<<<<<<<
 * 
 *     zst.zalloc = NULL
 */
  __pyx_v_block->next_out = (__pyx_v_block->next_out + (sizeof(__pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield)));

  /* "bgzip_utils.pyx":278
 *     block.next_out += sizeof(BlockHeaderBGZipSubfield)
 * 
 *     zst.zalloc = NULL             # <<<<<<<<<<<<<<
 *     zst.zfree = NULL
 *     zst.opaque = NULL
 */
  __pyx_v_zst.zalloc = NULL;

  /* "bgzip_utils.pyx":279
 * 
 *     zst.zalloc = NULL
 *     zst.zfree = NULL             # <<<<<<<<<<<<<<
 *     zst.opaque = NULL
 *     zst.next_in = block.next_in
 */
  __pyx_v_zst.zfree = NULL;

  /* "bgzip_utils.pyx":280
 *     zst.zalloc = NULL
 *     zst.zfree = NULL
 *     zst.opaque = NULL             # <<<<<<<<<<<<<<
 *     zst.next_in = block.next_in
 *     zst.avail_in = block.available_in
 */
  __pyx_v_zst.opaque = NULL;

  /* "bgzip_utils.pyx":281
 *     zst.zfree = NULL
 *     zst.opaque = NULL
 *     zst.next_in = block.next_in             # <<<<<<<<<<<<<<
 *     zst.avail_in = block.available_in
 *     zst.next_out = block.next_out
 */
  __pyx_t_1 = __pyx_v_block->next_in;
  __pyx_v_zst.next_in = __pyx_t_1;

  /* "bgzip_utils.pyx":282
 *     zst.opaque = NULL
 *     zst.next_in = block.next_in
 *     zst.avail_in = block.available_in             # <<<<<<<<<<<<<<
 *     zst.next_out = block.next_out
 *     zst.avail_out = 1024 * 1024
 */
  __pyx_t_2 = __pyx_v_block->available_in;
  __pyx_v_zst.avail_in = __pyx_t_2;

  /* "bgzip_utils.pyx":283
 *     zst.next_in = block.next_in
 *     zst.avail_in = block.available_in
 *     zst.next_out = block.next_out             # <<<<<<<<<<<<<<
 *     zst.avail_out = 1024 * 1024
 *     err = deflateInit2(&zst, Z_BEST_COMPRESSION, Z_DEFLATED, wbits, mem_level, Z_DEFAULT_STRATEGY)
 */
  __pyx_t_1 = __pyx_v_block->next_out;
  __pyx_v_zst.next_out = __pyx_t_1;

  /* "bgzip_utils.pyx":284
 *     zst.avail_in = block.available_in
 *     zst.next_out = block.next_out
 *     zst.avail_out = 1024 * 1024             # <<<<<<<<<<<<<<
 *     err = deflateInit2(&zst, Z_BEST_COMPRESSION, Z_DEFLATED, wbits, mem_level, Z_DEFAULT_STRATEGY)
 *     if Z_OK != err:
 */
  __pyx_v_zst.avail_out = 0x100000;

  /* "bgzip_utils.pyx":285
 *     zst.next_out = block.next_out
 *     zst.avail_out = 1024 * 1024
 *     err = deflateInit2(&zst, Z_BEST_COMPRESSION, Z_DEFLATED, wbits, mem_level, Z_DEFAULT_STRATEGY)             # <<<<<<<<<<<<<<
 *     if Z_OK != err:
 *         c_exception("Failed to initialize zlib compression")
 */
  __pyx_v_err = deflateInit2((&__pyx_v_zst), Z_BEST_COMPRESSION, Z_DEFLATED, __pyx_v_wbits, __pyx_v_mem_level, Z_DEFAULT_STRATEGY);

  /* "bgzip_utils.pyx":286
 *     zst.avail_out = 1024 * 1024
 *     err = deflateInit2(&zst, Z_BEST_COMPRESSION, Z_DEFLATED, wbits, mem_level, Z_DEFAULT_STRATEGY)
 *     if Z_OK != err:             # <<<<<<<<<<<<<<
 *         c_exception("Failed to initialize zlib compression")
 *     err = deflate(&zst, Z_FINISH)
 */
  __pyx_t_3 = ((Z_OK != __pyx_v_err) != 0);
  if (__pyx_t_3) {

    /* "bgzip_utils.pyx":287
 *     err = deflateInit2(&zst, Z_BEST_COMPRESSION, Z_DEFLATED, wbits, mem_level, Z_DEFAULT_STRATEGY)
 *     if Z_OK != err:
 *         c_exception("Failed to initialize zlib compression")             # <<<<<<<<<<<<<<
 *     err = deflate(&zst, Z_FINISH)
 *     if Z_STREAM_END != err:
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Failed to initialize zlib compression"));

    /* "bgzip_utils.pyx":286
 *     zst.avail_out = 1024 * 1024
 *     err = deflateInit2(&zst, Z_BEST_COMPRESSION, Z_DEFLATED, wbits, mem_level, Z_DEFAULT_STRATEGY)
 *     if Z_OK != err:             # <<<<<<<<<<<<<<
 *         c_exception("Failed to initialize zlib compression")
 *     err = deflate(&zst, Z_FINISH)
 */
  }

  /* "bgzip_utils.pyx":288
 *     if Z_OK != err:
 *         c_exception("Failed to initialize zlib compression")
 *     err = deflate(&zst, Z_FINISH)             # <<<<<<<<<<<<<<
 *     if Z_STREAM_END != err:
 *         c_exception("Compression error encountered")
 */
  __pyx_v_err = deflate((&__pyx_v_zst), Z_FINISH);

  /* "bgzip_utils.pyx":289
 *         c_exception("Failed to initialize zlib compression")
 *     err = deflate(&zst, Z_FINISH)
 *     if Z_STREAM_END != err:             # <<<<<<<<<<<<<<
 *         c_exception("Compression error encountered")
 *     deflateEnd(&zst)
 */
  __pyx_t_3 = ((Z_STREAM_END != __pyx_v_err) != 0);
  if (__pyx_t_3) {

    /* "bgzip_utils.pyx":290
 *     err = deflate(&zst, Z_FINISH)
 *     if Z_STREAM_END != err:
 *         c_exception("Compression error encountered")             # <<<<<<<<<<<<<<
 *     deflateEnd(&zst)
 * 
 */
    __pyx_f_11bgzip_utils_c_exception(((char const *)"Compression error encountered"));

    /* "bgzip_utils.pyx":289
 *         c_exception("Failed to initialize zlib compression")
 *     err = deflate(&zst, Z_FINISH)
 *     if Z_STREAM_END != err:             # <<<<<<<<<<<<<<
 *         c_exception("Compression error encountered")
 *     deflateEnd(&zst)
 */
  }

  /* "bgzip_utils.pyx":291
 *     if Z_STREAM_END != err:
 *         c_exception("Compression error encountered")
 *     deflateEnd(&zst)             # <<<<<<<<<<<<<<
 * 
 *     block.next_out += zst.total_out
 */
  (void)(deflateEnd((&__pyx_v_zst)));

  /* "bgzip_utils.pyx":293
 *     deflateEnd(&zst)
 * 
 *     block.next_out += zst.total_out             # <<<<<<<<<<<<<<
 * 
 *     tail = <BlockTailer *>block.next_out
 */
  __pyx_v_block->next_out = (__pyx_v_block->next_out + __pyx_v_zst.total_out);

  /* "bgzip_utils.pyx":295
 *     block.next_out += zst.total_out
 * 
 *     tail = <BlockTailer *>block.next_out             # <<<<<<<<<<<<<<
 * 
 *     for i in range(MAGIC_LENGTH):
 */
  __pyx_v_tail = ((__pyx_t_11bgzip_utils_BlockTailer *)__pyx_v_block->next_out);

  /* "bgzip_utils.pyx":297
 *     tail = <BlockTailer *>block.next_out
 * 
 *     for i in range(MAGIC_LENGTH):             # <<<<<<<<<<<<<<
 *         head.magic[i] = MAGIC[i]
 *     head.mod_time = 0
 */
  __pyx_t_4 = __pyx_e_11bgzip_utils_MAGIC_LENGTH;
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "bgzip_utils.pyx":298
 * 
 *     for i in range(MAGIC_LENGTH):
 *         head.magic[i] = MAGIC[i]             # <<<<<<<<<<<<<<
 *     head.mod_time = 0
 *     head.extra_flags = 0
 */
    (__pyx_v_head->magic[__pyx_v_i]) = (__pyx_v_11bgzip_utils_MAGIC[__pyx_v_i]);
  }

  /* "bgzip_utils.pyx":299
 *     for i in range(MAGIC_LENGTH):
 *         head.magic[i] = MAGIC[i]
 *     head.mod_time = 0             # <<<<<<<<<<<<<<
 *     head.extra_flags = 0
 *     head.os_type = b"\377"
 */
  __pyx_v_head->mod_time = 0;

  /* "bgzip_utils.pyx":300
 *         head.magic[i] = MAGIC[i]
 *     head.mod_time = 0
 *     head.extra_flags = 0             # <<<<<<<<<<<<<<
 *     head.os_type = b"\377"
 *     head.extra_len = sizeof(BlockHeaderBGZipSubfield)
 */
  __pyx_v_head->extra_flags = 0;

  /* "bgzip_utils.pyx":301
 *     head.mod_time = 0
 *     head.extra_flags = 0
 *     head.os_type = b"\377"             # <<<<<<<<<<<<<<
 *     head.extra_len = sizeof(BlockHeaderBGZipSubfield)
 * 
 */
  __pyx_v_head->os_type = '\xFF';

  /* "bgzip_utils.pyx":302
 *     head.extra_flags = 0
 *     head.os_type = b"\377"
 *     head.extra_len = sizeof(BlockHeaderBGZipSubfield)             # <<<<<<<<<<<<<<
 * 
 *     head_subfield.id_[0] = b"B"
 */
  __pyx_v_head->extra_len = (sizeof(__pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield));

  /* "bgzip_utils.pyx":304
 *     head.extra_len = sizeof(BlockHeaderBGZipSubfield)
 * 
 *     head_subfield.id_[0] = b"B"             # <<<<<<<<<<<<<<
 *     head_subfield.id_[1] = b"C"
 *     head_subfield.length = 2
 */
  (__pyx_v_head_subfield->id_[0]) = 'B';

  /* "bgzip_utils.pyx":305
 * 
 *     head_subfield.id_[0] = b"B"
 *     head_subfield.id_[1] = b"C"             # <<<<<<<<<<<<<<
 *     head_subfield.length = 2
 *     head_subfield.block_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + zst.total_out + sizeof(BlockTailer) - 1
 */
  (__pyx_v_head_subfield->id_[1]) = 'C';

  /* "bgzip_utils.pyx":306
 *     head_subfield.id_[0] = b"B"
 *     head_subfield.id_[1] = b"C"
 *     head_subfield.length = 2             # <<<<<<<<<<<<<<
 *     head_subfield.block_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + zst.total_out + sizeof(BlockTailer) - 1
 * 
 */
  __pyx_v_head_subfield->length = 2;

  /* "bgzip_utils.pyx":307
 *     head_subfield.id_[1] = b"C"
 *     head_subfield.length = 2
 *     head_subfield.block_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + zst.total_out + sizeof(BlockTailer) - 1             # <<<<<<<<<<<<<<
 * 
 *     tail.crc = crc32(0, block.next_in, block.inflated_size)
 */
  __pyx_v_head_subfield->block_size = (((((sizeof(__pyx_t_11bgzip_utils_BlockHeader)) + (sizeof(__pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield))) + __pyx_v_zst.total_out) + (sizeof(__pyx_t_11bgzip_utils_BlockTailer))) - 1);

  /* "bgzip_utils.pyx":309
 *     head_subfield.block_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + zst.total_out + sizeof(BlockTailer) - 1
 * 
 *     tail.crc = crc32(0, block.next_in, block.inflated_size)             # <<<<<<<<<<<<<<
 *     tail.inflated_size = block.inflated_size
 * 
 */
  __pyx_v_tail->crc = crc32(0, __pyx_v_block->next_in, __pyx_v_block->inflated_size);

  /* "bgzip_utils.pyx":310
 * 
 *     tail.crc = crc32(0, block.next_in, block.inflated_size)
 *     tail.inflated_size = block.inflated_size             # <<<<<<<<<<<<<<
 * 
 *     block.block_size = 1 + head_subfield.block_size
 */
  __pyx_t_2 = __pyx_v_block->inflated_size;
  __pyx_v_tail->inflated_size = __pyx_t_2;

  /* "bgzip_utils.pyx":312
 *     tail.inflated_size = block.inflated_size
 * 
 *     block.block_size = 1 + head_subfield.block_size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_block->block_size = (1 + __pyx_v_head_subfield->block_size);

  /* "bgzip_utils.pyx":263
 * 
 * 
 * cdef void compress_block(Block * block) nogil:             # <<<<<<<<<<<<<<
 *     cdef z_stream zst
 *     cdef int err = 0
 */

  /* function exit code */
}

/* "bgzip_utils.pyx":321
 * 
 * 
 * def compress_chunks(input_chunks, int num_threads):             # <<<<<<<<<<<<<<
 *     cdef int i, chunk_size, compressed_chunk_guess_size
 *     cdef int number_of_chunks = len(input_chunks)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11bgzip_utils_5compress_chunks(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_11bgzip_utils_5compress_chunks = {"compress_chunks", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_11bgzip_utils_5compress_chunks, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_11bgzip_utils_5compress_chunks(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_input_chunks = 0;
  CYTHON_UNUSED int __pyx_v_num_threads;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compress_chunks (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_input_chunks,&__pyx_n_s_num_threads,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_input_chunks)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_num_threads)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compress_chunks", 1, 2, 2, 1); __PYX_ERR(0, 321, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compress_chunks") < 0)) __PYX_ERR(0, 321, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_input_chunks = values[0];
    __pyx_v_num_threads = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_num_threads == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 321, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compress_chunks", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 321, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bgzip_utils.compress_chunks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11bgzip_utils_4compress_chunks(__pyx_self, __pyx_v_input_chunks, __pyx_v_num_threads);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11bgzip_utils_4compress_chunks(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_input_chunks, CYTHON_UNUSED int __pyx_v_num_threads) {
  int __pyx_v_i;
  int __pyx_v_chunk_size;
  int __pyx_v_compressed_chunk_guess_size;
  int __pyx_v_number_of_chunks;
  __pyx_t_11bgzip_utils_Block __pyx_v_blocks[__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS];
  PyObject *__pyx_v_chunk;
  PyObject *__pyx_v_compressed_chunk;
  PyObject *__pyx_v_chunks;
  PyObject *__pyx_v_compressed_chunks[__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS];
  PyObject *__pyx_v_ret = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_UCS4 __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("compress_chunks", 0);

  /* "bgzip_utils.pyx":323
 * def compress_chunks(input_chunks, int num_threads):
 *     cdef int i, chunk_size, compressed_chunk_guess_size
 *     cdef int number_of_chunks = len(input_chunks)             # <<<<<<<<<<<<<<
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_input_chunks); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 323, __pyx_L1_error)
  __pyx_v_number_of_chunks = __pyx_t_1;

  /* "bgzip_utils.pyx":326
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 *     if number_of_chunks > NUMBER_OF_BLOCKS:             # <<<<<<<<<<<<<<
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")
 * 
 */
  __pyx_t_2 = ((__pyx_v_number_of_chunks > __pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "bgzip_utils.pyx":327
 * 
 *     if number_of_chunks > NUMBER_OF_BLOCKS:
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject * chunk, * compressed_chunk
 */
    __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_4 = 127;
    __Pyx_INCREF(__pyx_kp_u_Cannot_compress_more_than);
    __pyx_t_1 += 26;
    __Pyx_GIVEREF(__pyx_kp_u_Cannot_compress_more_than);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_Cannot_compress_more_than);
    __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_kp_u_chunks_per_call_Received);
    __pyx_t_1 += 27;
    __Pyx_GIVEREF(__pyx_kp_u_chunks_per_call_Received);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u_chunks_per_call_Received);
    __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_v_number_of_chunks, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_3, 4, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 327, __pyx_L1_error)

    /* "bgzip_utils.pyx":326
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 *     if number_of_chunks > NUMBER_OF_BLOCKS:             # <<<<<<<<<<<<<<
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")
 * 
 */
  }

  /* "bgzip_utils.pyx":330
 * 
 *     cdef PyObject * chunk, * compressed_chunk
 *     cdef PyObject * chunks = <PyObject *>input_chunks             # <<<<<<<<<<<<<<
 *     cdef PyObject * compressed_chunks[NUMBER_OF_BLOCKS]
 * 
 */
  __pyx_v_chunks = ((PyObject *)__pyx_v_input_chunks);

  /* "bgzip_utils.pyx":333
 *     cdef PyObject * compressed_chunks[NUMBER_OF_BLOCKS]
 * 
 *     for i in range(number_of_chunks):             # <<<<<<<<<<<<<<
 *         chunk = PyList_GetItem(chunks, i)
 *         chunk_size = PyBytes_GET_SIZE(chunk)
 */
  __pyx_t_6 = __pyx_v_number_of_chunks;
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "bgzip_utils.pyx":334
 * 
 *     for i in range(number_of_chunks):
 *         chunk = PyList_GetItem(chunks, i)             # <<<<<<<<<<<<<<
 *         chunk_size = PyBytes_GET_SIZE(chunk)
 *         compressed_chunk_guess_size = chunk_size + _block_metadata_size
 */
    __pyx_v_chunk = PyList_GetItem(__pyx_v_chunks, __pyx_v_i);

    /* "bgzip_utils.pyx":335
 *     for i in range(number_of_chunks):
 *         chunk = PyList_GetItem(chunks, i)
 *         chunk_size = PyBytes_GET_SIZE(chunk)             # <<<<<<<<<<<<<<
 *         compressed_chunk_guess_size = chunk_size + _block_metadata_size
 *         compressed_chunk = PyByteArray_FromStringAndSize(NULL, compressed_chunk_guess_size)
 */
    __pyx_v_chunk_size = PyBytes_GET_SIZE(__pyx_v_chunk);

    /* "bgzip_utils.pyx":336
 *         chunk = PyList_GetItem(chunks, i)
 *         chunk_size = PyBytes_GET_SIZE(chunk)
 *         compressed_chunk_guess_size = chunk_size + _block_metadata_size             # <<<<<<<<<<<<<<
 *         compressed_chunk = PyByteArray_FromStringAndSize(NULL, compressed_chunk_guess_size)
 *         compressed_chunks[i] = compressed_chunk
 */
    __pyx_v_compressed_chunk_guess_size = (__pyx_v_chunk_size + __pyx_v_11bgzip_utils__block_metadata_size);

    /* "bgzip_utils.pyx":337
 *         chunk_size = PyBytes_GET_SIZE(chunk)
 *         compressed_chunk_guess_size = chunk_size + _block_metadata_size
 *         compressed_chunk = PyByteArray_FromStringAndSize(NULL, compressed_chunk_guess_size)             # <<<<<<<<<<<<<<
 *         compressed_chunks[i] = compressed_chunk
 * 
 */
    __pyx_v_compressed_chunk = PyByteArray_FromStringAndSize(NULL, __pyx_v_compressed_chunk_guess_size);

    /* "bgzip_utils.pyx":338
 *         compressed_chunk_guess_size = chunk_size + _block_metadata_size
 *         compressed_chunk = PyByteArray_FromStringAndSize(NULL, compressed_chunk_guess_size)
 *         compressed_chunks[i] = compressed_chunk             # <<<<<<<<<<<<<<
 * 
 *         blocks[i].inflated_size = chunk_size
 */
    (__pyx_v_compressed_chunks[__pyx_v_i]) = __pyx_v_compressed_chunk;

    /* "bgzip_utils.pyx":340
 *         compressed_chunks[i] = compressed_chunk
 * 
 *         blocks[i].inflated_size = chunk_size             # <<<<<<<<<<<<<<
 *         blocks[i].next_in = <Bytef *>PyBytes_AS_STRING(chunk)
 *         blocks[i].available_in = chunk_size
 */
    (__pyx_v_blocks[__pyx_v_i]).inflated_size = __pyx_v_chunk_size;

    /* "bgzip_utils.pyx":341
 * 
 *         blocks[i].inflated_size = chunk_size
 *         blocks[i].next_in = <Bytef *>PyBytes_AS_STRING(chunk)             # <<<<<<<<<<<<<<
 *         blocks[i].available_in = chunk_size
 *         blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)
 */
    (__pyx_v_blocks[__pyx_v_i]).next_in = ((__pyx_t_5czlib_Bytef *)PyBytes_AS_STRING(__pyx_v_chunk));

    /* "bgzip_utils.pyx":342
 *         blocks[i].inflated_size = chunk_size
 *         blocks[i].next_in = <Bytef *>PyBytes_AS_STRING(chunk)
 *         blocks[i].available_in = chunk_size             # <<<<<<<<<<<<<<
 *         blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)
 *         blocks[i].avail_out = compressed_chunk_guess_size
 */
    (__pyx_v_blocks[__pyx_v_i]).available_in = __pyx_v_chunk_size;

    /* "bgzip_utils.pyx":343
 *         blocks[i].next_in = <Bytef *>PyBytes_AS_STRING(chunk)
 *         blocks[i].available_in = chunk_size
 *         blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)             # <<<<<<<<<<<<<<
 *         blocks[i].avail_out = compressed_chunk_guess_size
 * 
 */
    (__pyx_v_blocks[__pyx_v_i]).next_out = ((__pyx_t_5czlib_Bytef *)PyByteArray_AS_STRING(__pyx_v_compressed_chunk));

    /* "bgzip_utils.pyx":344
 *         blocks[i].available_in = chunk_size
 *         blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)
 *         blocks[i].avail_out = compressed_chunk_guess_size             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    (__pyx_v_blocks[__pyx_v_i]).avail_out = __pyx_v_compressed_chunk_guess_size;
  }

  /* "bgzip_utils.pyx":346
 *         blocks[i].avail_out = compressed_chunk_guess_size
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):
 *             compress_block(&blocks[i])
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "bgzip_utils.pyx":347
 * 
 *     with nogil:
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):             # <<<<<<<<<<<<<<
 *             compress_block(&blocks[i])
 *             compressed_chunk = compressed_chunks[i]
 */
        __pyx_t_6 = __pyx_v_number_of_chunks;
        if (1 == 0) abort();
        {
            #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
                #undef likely
                #undef unlikely
                #define likely(x)   (x)
                #define unlikely(x) (x)
            #endif
            __pyx_t_8 = (__pyx_t_6 - 0 + 1 - 1/abs(1)) / 1;
            if (__pyx_t_8 > 0)
            {
                #ifdef _OPENMP
                #pragma omp parallel num_threads(__pyx_v_num_threads)
                #endif /* _OPENMP */
                {
                    #ifdef _OPENMP
                    #pragma omp for lastprivate(__pyx_v_compressed_chunk) firstprivate(__pyx_v_i) lastprivate(__pyx_v_i) schedule(dynamic)
                    #endif /* _OPENMP */
                    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_8; __pyx_t_7++){
                        {
                            __pyx_v_i = (int)(0 + 1 * __pyx_t_7);
                            /* Initialize private variables to invalid values */
                            __pyx_v_compressed_chunk = ((PyObject *)1);

                            /* "bgzip_utils.pyx":348
 *     with nogil:
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):
 *             compress_block(&blocks[i])             # <<<<<<<<<<<<<<
 *             compressed_chunk = compressed_chunks[i]
 * 
 */
                            __pyx_f_11bgzip_utils_compress_block((&(__pyx_v_blocks[__pyx_v_i])));

                            /* "bgzip_utils.pyx":349
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):
 *             compress_block(&blocks[i])
 *             compressed_chunk = compressed_chunks[i]             # <<<<<<<<<<<<<<
 * 
 *     ret = list()
 */
                            __pyx_v_compressed_chunk = (__pyx_v_compressed_chunks[__pyx_v_i]);
                        }
                    }
                }
            }
        }
        #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
            #undef likely
            #undef unlikely
            #define likely(x)   __builtin_expect(!!(x), 1)
            #define unlikely(x) __builtin_expect(!!(x), 0)
        #endif
      }

      /* "bgzip_utils.pyx":346
 *         blocks[i].avail_out = compressed_chunk_guess_size
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):
 *             compress_block(&blocks[i])
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "bgzip_utils.pyx":351
 *             compressed_chunk = compressed_chunks[i]
 * 
 *     ret = list()             # <<<<<<<<<<<<<<
 *     for i in range(number_of_chunks):
 *         compressed_chunk = compressed_chunks[i]
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_ret = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "bgzip_utils.pyx":352
 * 
 *     ret = list()
 *     for i in range(number_of_chunks):             # <<<<<<<<<<<<<<
 *         compressed_chunk = compressed_chunks[i]
 *         PyByteArray_Resize(compressed_chunk, blocks[i].block_size)
 */
  __pyx_t_8 = __pyx_v_number_of_chunks;
  __pyx_t_7 = __pyx_t_8;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_7; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "bgzip_utils.pyx":353
 *     ret = list()
 *     for i in range(number_of_chunks):
 *         compressed_chunk = compressed_chunks[i]             # <<<<<<<<<<<<<<
 *         PyByteArray_Resize(compressed_chunk, blocks[i].block_size)
 *         ret.append(<bytearray>compressed_chunks[i])
 */
    __pyx_v_compressed_chunk = (__pyx_v_compressed_chunks[__pyx_v_i]);

    /* "bgzip_utils.pyx":354
 *     for i in range(number_of_chunks):
 *         compressed_chunk = compressed_chunks[i]
 *         PyByteArray_Resize(compressed_chunk, blocks[i].block_size)             # <<<<<<<<<<<<<<
 *         ret.append(<bytearray>compressed_chunks[i])
 *         Py_DECREF(compressed_chunk)
 */
    (void)(PyByteArray_Resize(__pyx_v_compressed_chunk, (__pyx_v_blocks[__pyx_v_i]).block_size));

    /* "bgzip_utils.pyx":355
 *         compressed_chunk = compressed_chunks[i]
 *         PyByteArray_Resize(compressed_chunk, blocks[i].block_size)
 *         ret.append(<bytearray>compressed_chunks[i])             # <<<<<<<<<<<<<<
 *         Py_DECREF(compressed_chunk)
 *     return ret
 */
    __pyx_t_3 = ((PyObject *)(__pyx_v_compressed_chunks[__pyx_v_i]));
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_t_3); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "bgzip_utils.pyx":356
 *         PyByteArray_Resize(compressed_chunk, blocks[i].block_size)
 *         ret.append(<bytearray>compressed_chunks[i])
 *         Py_DECREF(compressed_chunk)             # <<<<<<<<<<<<<<
 *     return ret
 * 
 */
    Py_DECREF(__pyx_v_compressed_chunk);
  }

  /* "bgzip_utils.pyx":357
 *         ret.append(<bytearray>compressed_chunks[i])
 *         Py_DECREF(compressed_chunk)
 *     return ret             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ret);
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "bgzip_utils.pyx":321
 * 
 * 
 * def compress_chunks(input_chunks, int num_threads):             # <<<<<<<<<<<<<<
 *     cdef int i, chunk_size, compressed_chunk_guess_size
 *     cdef int number_of_chunks = len(input_chunks)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bgzip_utils.compress_chunks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bgzip_utils.pyx":360
 * 
 * 
 * cdef void _get_buffer(PyObject * obj, Py_buffer * view):             # <<<<<<<<<<<<<<
 *     cdef int err
 * 
 */

static void __pyx_f_11bgzip_utils__get_buffer(PyObject *__pyx_v_obj, Py_buffer *__pyx_v_view) {
  int __pyx_v_err;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_get_buffer", 0);

  /* "bgzip_utils.pyx":363
 *     cdef int err
 * 
 *     err = PyObject_GetBuffer(obj, view, PyBUF_SIMPLE)             # <<<<<<<<<<<<<<
 *     if -1 == err:
 *         raise Exception()
 */
  __pyx_v_err = PyObject_GetBuffer(__pyx_v_obj, __pyx_v_view, PyBUF_SIMPLE);

  /* "bgzip_utils.pyx":364
 * 
 *     err = PyObject_GetBuffer(obj, view, PyBUF_SIMPLE)
 *     if -1 == err:             # <<<<<<<<<<<<<<
 *         raise Exception()
 * 
 */
  __pyx_t_1 = ((-1L == __pyx_v_err) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "bgzip_utils.pyx":365
 *     err = PyObject_GetBuffer(obj, view, PyBUF_SIMPLE)
 *     if -1 == err:
 *         raise Exception()             # <<<<<<<<<<<<<<
 * 
 * def compress_to_stream(input_buff_obj, list scratch_buffers, handle, int num_threads):
 */
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0]))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 365, __pyx_L1_error)

    /* "bgzip_utils.pyx":364
 * 
 *     err = PyObject_GetBuffer(obj, view, PyBUF_SIMPLE)
 *     if -1 == err:             # <<<<<<<<<<<<<<
 *         raise Exception()
 * 
 */
  }

  /* "bgzip_utils.pyx":360
 * 
 * 
 * cdef void _get_buffer(PyObject * obj, Py_buffer * view):             # <<<<<<<<<<<<<<
 *     cdef int err
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("bgzip_utils._get_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "bgzip_utils.pyx":367
 *         raise Exception()
 * 
 * def compress_to_stream(input_buff_obj, list scratch_buffers, handle, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Compress the data in `input_buff_obj` and write it to `handle`.
 */

/* Python wrapper */
static PyObject *__pyx_pw_11bgzip_utils_7compress_to_stream(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11bgzip_utils_6compress_to_stream[] = "\n    Compress the data in `input_buff_obj` and write it to `handle`.\n\n    `scratch_buffers` should contain enough buffers to hold the number of blocks compressed. Each\n    buffer should hold `_block_inflated_size + _block_metadata_size` bytes.\n    ";
static PyMethodDef __pyx_mdef_11bgzip_utils_7compress_to_stream = {"compress_to_stream", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_11bgzip_utils_7compress_to_stream, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11bgzip_utils_6compress_to_stream};
static PyObject *__pyx_pw_11bgzip_utils_7compress_to_stream(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_input_buff_obj = 0;
  PyObject *__pyx_v_scratch_buffers = 0;
  PyObject *__pyx_v_handle = 0;
  CYTHON_UNUSED int __pyx_v_num_threads;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compress_to_stream (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_input_buff_obj,&__pyx_n_s_scratch_buffers,&__pyx_n_s_handle,&__pyx_n_s_num_threads,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_input_buff_obj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_scratch_buffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compress_to_stream", 1, 4, 4, 1); __PYX_ERR(0, 367, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compress_to_stream", 1, 4, 4, 2); __PYX_ERR(0, 367, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_num_threads)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compress_to_stream", 1, 4, 4, 3); __PYX_ERR(0, 367, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compress_to_stream") < 0)) __PYX_ERR(0, 367, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_input_buff_obj = values[0];
    __pyx_v_scratch_buffers = ((PyObject*)values[1]);
    __pyx_v_handle = values[2];
    __pyx_v_num_threads = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_num_threads == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compress_to_stream", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 367, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("bgzip_utils.compress_to_stream", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_scratch_buffers), (&PyList_Type), 1, "scratch_buffers", 1))) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_r = __pyx_pf_11bgzip_utils_6compress_to_stream(__pyx_self, __pyx_v_input_buff_obj, __pyx_v_scratch_buffers, __pyx_v_handle, __pyx_v_num_threads);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11bgzip_utils_6compress_to_stream(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_input_buff_obj, PyObject *__pyx_v_scratch_buffers, PyObject *__pyx_v_handle, CYTHON_UNUSED int __pyx_v_num_threads) {
  int __pyx_v_i;
  int __pyx_v_chunk_size;
  unsigned int __pyx_v_bytes_available;
  int __pyx_v_number_of_chunks;
  __pyx_t_11bgzip_utils_Block __pyx_v_blocks[__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS];
  PyObject *__pyx_v_compressed_chunks;
  PyObject *__pyx_v_compressed_chunk;
  Py_buffer __pyx_v_input_view;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_UCS4 __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("compress_to_stream", 0);

  /* "bgzip_utils.pyx":375
 *     """
 *     cdef int i, chunk_size
 *     cdef unsigned int bytes_available = len(input_buff_obj)             # <<<<<<<<<<<<<<
 *     cdef int number_of_chunks = ceil(bytes_available / block_inflated_size)
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_input_buff_obj); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 375, __pyx_L1_error)
  __pyx_v_bytes_available = __pyx_t_1;

  /* "bgzip_utils.pyx":376
 *     cdef int i, chunk_size
 *     cdef unsigned int bytes_available = len(input_buff_obj)
 *     cdef int number_of_chunks = ceil(bytes_available / block_inflated_size)             # <<<<<<<<<<<<<<
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_ceil); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_bytes_available); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_block_inflated_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyNumber_Divide(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_number_of_chunks = __pyx_t_7;

  /* "bgzip_utils.pyx":379
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 *     if number_of_chunks > NUMBER_OF_BLOCKS:             # <<<<<<<<<<<<<<
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")
 * 
 */
  __pyx_t_8 = ((__pyx_v_number_of_chunks > __pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS) != 0);
  if (unlikely(__pyx_t_8)) {

    /* "bgzip_utils.pyx":380
 * 
 *     if number_of_chunks > NUMBER_OF_BLOCKS:
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject * compressed_chunks = <PyObject *>scratch_buffers
 */
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = 0;
    __pyx_t_9 = 127;
    __Pyx_INCREF(__pyx_kp_u_Cannot_compress_more_than);
    __pyx_t_1 += 26;
    __Pyx_GIVEREF(__pyx_kp_u_Cannot_compress_more_than);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Cannot_compress_more_than);
    __pyx_t_3 = __Pyx_PyUnicode_From_int(__pyx_e_11bgzip_utils_NUMBER_OF_BLOCKS, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u_chunks_per_call_Received);
    __pyx_t_1 += 27;
    __Pyx_GIVEREF(__pyx_kp_u_chunks_per_call_Received);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_chunks_per_call_Received);
    __pyx_t_3 = __Pyx_PyUnicode_From_int(__pyx_v_number_of_chunks, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_2, 4, __pyx_t_1, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 380, __pyx_L1_error)

    /* "bgzip_utils.pyx":379
 *     cdef Block blocks[NUMBER_OF_BLOCKS]
 * 
 *     if number_of_chunks > NUMBER_OF_BLOCKS:             # <<<<<<<<<<<<<<
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")
 * 
 */
  }

  /* "bgzip_utils.pyx":382
 *         raise Exception(f"Cannot compress more than {NUMBER_OF_BLOCKS} chunks per call. Received {number_of_chunks}")
 * 
 *     cdef PyObject * compressed_chunks = <PyObject *>scratch_buffers             # <<<<<<<<<<<<<<
 *     cdef PyObject * compressed_chunk
 * 
 */
  __pyx_v_compressed_chunks = ((PyObject *)__pyx_v_scratch_buffers);

  /* "bgzip_utils.pyx":386
 * 
 *     cdef Py_buffer input_view
 *     _get_buffer(<PyObject *>input_buff_obj, &input_view)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_f_11bgzip_utils__get_buffer(((PyObject *)__pyx_v_input_buff_obj), (&__pyx_v_input_view));

  /* "bgzip_utils.pyx":388
 *     _get_buffer(<PyObject *>input_buff_obj, &input_view)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(number_of_chunks):
 *             compressed_chunk = <PyObject *>PyList_GetItem(compressed_chunks, i)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "bgzip_utils.pyx":389
 * 
 *     with nogil:
 *         for i in range(number_of_chunks):             # <<<<<<<<<<<<<<
 *             compressed_chunk = <PyObject *>PyList_GetItem(compressed_chunks, i)
 * 
 */
        __pyx_t_7 = __pyx_v_number_of_chunks;
        __pyx_t_10 = __pyx_t_7;
        for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
          __pyx_v_i = __pyx_t_11;

          /* "bgzip_utils.pyx":390
 *     with nogil:
 *         for i in range(number_of_chunks):
 *             compressed_chunk = <PyObject *>PyList_GetItem(compressed_chunks, i)             # <<<<<<<<<<<<<<
 * 
 *             if bytes_available >= _block_inflated_size:
 */
          __pyx_v_compressed_chunk = ((PyObject *)PyList_GetItem(__pyx_v_compressed_chunks, __pyx_v_i));

          /* "bgzip_utils.pyx":392
 *             compressed_chunk = <PyObject *>PyList_GetItem(compressed_chunks, i)
 * 
 *             if bytes_available >= _block_inflated_size:             # <<<<<<<<<<<<<<
 *                 chunk_size = _block_inflated_size
 *             else:
 */
          __pyx_t_8 = ((__pyx_v_bytes_available >= __pyx_v_11bgzip_utils__block_inflated_size) != 0);
          if (__pyx_t_8) {

            /* "bgzip_utils.pyx":393
 * 
 *             if bytes_available >= _block_inflated_size:
 *                 chunk_size = _block_inflated_size             # <<<<<<<<<<<<<<
 *             else:
 *                 chunk_size = bytes_available
 */
            __pyx_v_chunk_size = __pyx_v_11bgzip_utils__block_inflated_size;

            /* "bgzip_utils.pyx":392
 *             compressed_chunk = <PyObject *>PyList_GetItem(compressed_chunks, i)
 * 
 *             if bytes_available >= _block_inflated_size:             # <<<<<<<<<<<<<<
 *                 chunk_size = _block_inflated_size
 *             else:
 */
            goto __pyx_L9;
          }

          /* "bgzip_utils.pyx":395
 *                 chunk_size = _block_inflated_size
 *             else:
 *                 chunk_size = bytes_available             # <<<<<<<<<<<<<<
 * 
 *             bytes_available -= _block_inflated_size
 */
          /*else*/ {
            __pyx_v_chunk_size = __pyx_v_bytes_available;
          }
          __pyx_L9:;

          /* "bgzip_utils.pyx":397
 *                 chunk_size = bytes_available
 * 
 *             bytes_available -= _block_inflated_size             # <<<<<<<<<<<<<<
 * 
 *             blocks[i].inflated_size = chunk_size
 */
          __pyx_v_bytes_available = (__pyx_v_bytes_available - __pyx_v_11bgzip_utils__block_inflated_size);

          /* "bgzip_utils.pyx":399
 *             bytes_available -= _block_inflated_size
 * 
 *             blocks[i].inflated_size = chunk_size             # <<<<<<<<<<<<<<
 *             blocks[i].next_in = <Bytef *>input_view.buf + (i * _block_inflated_size)
 *             blocks[i].available_in = chunk_size
 */
          (__pyx_v_blocks[__pyx_v_i]).inflated_size = __pyx_v_chunk_size;

          /* "bgzip_utils.pyx":400
 * 
 *             blocks[i].inflated_size = chunk_size
 *             blocks[i].next_in = <Bytef *>input_view.buf + (i * _block_inflated_size)             # <<<<<<<<<<<<<<
 *             blocks[i].available_in = chunk_size
 *             blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)
 */
          (__pyx_v_blocks[__pyx_v_i]).next_in = (((__pyx_t_5czlib_Bytef *)__pyx_v_input_view.buf) + (__pyx_v_i * __pyx_v_11bgzip_utils__block_inflated_size));

          /* "bgzip_utils.pyx":401
 *             blocks[i].inflated_size = chunk_size
 *             blocks[i].next_in = <Bytef *>input_view.buf + (i * _block_inflated_size)
 *             blocks[i].available_in = chunk_size             # <<<<<<<<<<<<<<
 *             blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)
 *             blocks[i].avail_out = _block_inflated_size + _block_metadata_size
 */
          (__pyx_v_blocks[__pyx_v_i]).available_in = __pyx_v_chunk_size;

          /* "bgzip_utils.pyx":402
 *             blocks[i].next_in = <Bytef *>input_view.buf + (i * _block_inflated_size)
 *             blocks[i].available_in = chunk_size
 *             blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)             # <<<<<<<<<<<<<<
 *             blocks[i].avail_out = _block_inflated_size + _block_metadata_size
 * 
 */
          (__pyx_v_blocks[__pyx_v_i]).next_out = ((__pyx_t_5czlib_Bytef *)PyByteArray_AS_STRING(__pyx_v_compressed_chunk));

          /* "bgzip_utils.pyx":403
 *             blocks[i].available_in = chunk_size
 *             blocks[i].next_out = <Bytef *>PyByteArray_AS_STRING(compressed_chunk)
 *             blocks[i].avail_out = _block_inflated_size + _block_metadata_size             # <<<<<<<<<<<<<<
 * 
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):
 */
          (__pyx_v_blocks[__pyx_v_i]).avail_out = (__pyx_v_11bgzip_utils__block_inflated_size + __pyx_v_11bgzip_utils__block_metadata_size);
        }

        /* "bgzip_utils.pyx":405
 *             blocks[i].avail_out = _block_inflated_size + _block_metadata_size
 * 
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):             # <<<<<<<<<<<<<<
 *             compress_block(&blocks[i])
 * 
 */
        __pyx_t_7 = __pyx_v_number_of_chunks;
        if (1 == 0) abort();
        {
            #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
                #undef likely
                #undef unlikely
                #define likely(x)   (x)
                #define unlikely(x) (x)
            #endif
            __pyx_t_11 = (__pyx_t_7 - 0 + 1 - 1/abs(1)) / 1;
            if (__pyx_t_11 > 0)
            {
                #ifdef _OPENMP
                #pragma omp parallel num_threads(__pyx_v_num_threads)
                #endif /* _OPENMP */
                {
                    #ifdef _OPENMP
                    #pragma omp for firstprivate(__pyx_v_i) lastprivate(__pyx_v_i) schedule(dynamic)
                    #endif /* _OPENMP */
                    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_11; __pyx_t_10++){
                        {
                            __pyx_v_i = (int)(0 + 1 * __pyx_t_10);

                            /* "bgzip_utils.pyx":406
 * 
 *         for i in prange(number_of_chunks, num_threads=num_threads, schedule="dynamic"):
 *             compress_block(&blocks[i])             # <<<<<<<<<<<<<<
 * 
 *     PyBuffer_Release(&input_view)
 */
                            __pyx_f_11bgzip_utils_compress_block((&(__pyx_v_blocks[__pyx_v_i])));
                        }
                    }
                }
            }
        }
        #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
            #undef likely
            #undef unlikely
            #define likely(x)   __builtin_expect(!!(x), 1)
            #define unlikely(x) __builtin_expect(!!(x), 0)
        #endif
      }

      /* "bgzip_utils.pyx":388
 *     _get_buffer(<PyObject *>input_buff_obj, &input_view)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(number_of_chunks):
 *             compressed_chunk = <PyObject *>PyList_GetItem(compressed_chunks, i)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "bgzip_utils.pyx":408
 *             compress_block(&blocks[i])
 * 
 *     PyBuffer_Release(&input_view)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(number_of_chunks):
 */
  PyBuffer_Release((&__pyx_v_input_view));

  /* "bgzip_utils.pyx":410
 *     PyBuffer_Release(&input_view)
 * 
 *     for i in range(number_of_chunks):             # <<<<<<<<<<<<<<
 *         handle.write(scratch_buffers[i][:blocks[i].block_size])
 */
  __pyx_t_11 = __pyx_v_number_of_chunks;
  __pyx_t_10 = __pyx_t_11;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_10; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "bgzip_utils.pyx":411
 * 
 *     for i in range(number_of_chunks):
 *         handle.write(scratch_buffers[i][:blocks[i].block_size])             # <<<<<<<<<<<<<<
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_handle, __pyx_n_s_write); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(__pyx_v_scratch_buffers == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 411, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_scratch_buffers, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_6, 0, (__pyx_v_blocks[__pyx_v_i]).block_size, NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "bgzip_utils.pyx":367
 *         raise Exception()
 * 
 * def compress_to_stream(input_buff_obj, list scratch_buffers, handle, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Compress the data in `input_buff_obj` and write it to `handle`.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("bgzip_utils.compress_to_stream", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_bgzip_utils(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_bgzip_utils},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "bgzip_utils",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_b_BC, __pyx_k_BC, sizeof(__pyx_k_BC), 0, 0, 0, 0},
  {&__pyx_kp_u_Cannot_compress_more_than, __pyx_k_Cannot_compress_more_than, sizeof(__pyx_k_Cannot_compress_more_than), 0, 1, 0, 0},
  {&__pyx_n_s_avail_out, __pyx_k_avail_out, sizeof(__pyx_k_avail_out), 0, 0, 1, 1},
  {&__pyx_n_s_bgzip_utils, __pyx_k_bgzip_utils, sizeof(__pyx_k_bgzip_utils), 0, 0, 1, 1},
  {&__pyx_kp_s_bgzip_utils_bgzip_utils_pyx, __pyx_k_bgzip_utils_bgzip_utils_pyx, sizeof(__pyx_k_bgzip_utils_bgzip_utils_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_block_inflated_size, __pyx_k_block_inflated_size, sizeof(__pyx_k_block_inflated_size), 0, 0, 1, 1},
  {&__pyx_n_s_block_metadata_size, __pyx_k_block_metadata_size, sizeof(__pyx_k_block_metadata_size), 0, 0, 1, 1},
  {&__pyx_n_s_blocks, __pyx_k_blocks, sizeof(__pyx_k_blocks), 0, 0, 1, 1},
  {&__pyx_n_s_bytes_available, __pyx_k_bytes_available, sizeof(__pyx_k_bytes_available), 0, 0, 1, 1},
  {&__pyx_n_s_bytes_inflated, __pyx_k_bytes_inflated, sizeof(__pyx_k_bytes_inflated), 0, 0, 1, 1},
  {&__pyx_n_s_bytes_read, __pyx_k_bytes_read, sizeof(__pyx_k_bytes_read), 0, 0, 1, 1},
  {&__pyx_n_s_ceil, __pyx_k_ceil, sizeof(__pyx_k_ceil), 0, 0, 1, 1},
  {&__pyx_n_s_chunk, __pyx_k_chunk, sizeof(__pyx_k_chunk), 0, 0, 1, 1},
  {&__pyx_n_s_chunk_size, __pyx_k_chunk_size, sizeof(__pyx_k_chunk_size), 0, 0, 1, 1},
  {&__pyx_n_s_chunks, __pyx_k_chunks, sizeof(__pyx_k_chunks), 0, 0, 1, 1},
  {&__pyx_kp_u_chunks_per_call_Received, __pyx_k_chunks_per_call_Received, sizeof(__pyx_k_chunks_per_call_Received), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_compress_chunks, __pyx_k_compress_chunks, sizeof(__pyx_k_compress_chunks), 0, 0, 1, 1},
  {&__pyx_n_s_compress_to_stream, __pyx_k_compress_to_stream, sizeof(__pyx_k_compress_to_stream), 0, 0, 1, 1},
  {&__pyx_n_s_compressed_chunk, __pyx_k_compressed_chunk, sizeof(__pyx_k_compressed_chunk), 0, 0, 1, 1},
  {&__pyx_n_s_compressed_chunk_guess_size, __pyx_k_compressed_chunk_guess_size, sizeof(__pyx_k_compressed_chunk_guess_size), 0, 0, 1, 1},
  {&__pyx_n_s_compressed_chunks, __pyx_k_compressed_chunks, sizeof(__pyx_k_compressed_chunks), 0, 0, 1, 1},
  {&__pyx_n_s_decompress_into, __pyx_k_decompress_into, sizeof(__pyx_k_decompress_into), 0, 0, 1, 1},
  {&__pyx_n_s_decompress_into_2, __pyx_k_decompress_into_2, sizeof(__pyx_k_decompress_into_2), 0, 0, 1, 1},
  {&__pyx_n_s_dst, __pyx_k_dst, sizeof(__pyx_k_dst), 0, 0, 1, 1},
  {&__pyx_n_s_dst_buff, __pyx_k_dst_buff, sizeof(__pyx_k_dst_buff), 0, 0, 1, 1},
  {&__pyx_n_s_dst_buff_obj, __pyx_k_dst_buff_obj, sizeof(__pyx_k_dst_buff_obj), 0, 0, 1, 1},
  {&__pyx_kp_s_dst_buff_obj_must_be_a_memoryvie, __pyx_k_dst_buff_obj_must_be_a_memoryvie, sizeof(__pyx_k_dst_buff_obj_must_be_a_memoryvie), 0, 0, 1, 0},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_inflated_size, __pyx_k_inflated_size, sizeof(__pyx_k_inflated_size), 0, 0, 1, 1},
  {&__pyx_n_s_input_buff_obj, __pyx_k_input_buff_obj, sizeof(__pyx_k_input_buff_obj), 0, 0, 1, 1},
  {&__pyx_n_s_input_chunks, __pyx_k_input_chunks, sizeof(__pyx_k_input_chunks), 0, 0, 1, 1},
  {&__pyx_n_s_input_view, __pyx_k_input_view, sizeof(__pyx_k_input_view), 0, 0, 1, 1},
  {&__pyx_n_s_io, __pyx_k_io, sizeof(__pyx_k_io), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_num_threads, __pyx_k_num_threads, sizeof(__pyx_k_num_threads), 0, 0, 1, 1},
  {&__pyx_n_s_number_of_blocks, __pyx_k_number_of_blocks, sizeof(__pyx_k_number_of_blocks), 0, 0, 1, 1},
  {&__pyx_n_s_number_of_chunks, __pyx_k_number_of_chunks, sizeof(__pyx_k_number_of_chunks), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_out, __pyx_k_out, sizeof(__pyx_k_out), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_ret, __pyx_k_ret, sizeof(__pyx_k_ret), 0, 0, 1, 1},
  {&__pyx_n_s_scratch_buffers, __pyx_k_scratch_buffers, sizeof(__pyx_k_scratch_buffers), 0, 0, 1, 1},
  {&__pyx_n_s_src, __pyx_k_src, sizeof(__pyx_k_src), 0, 0, 1, 1},
  {&__pyx_n_s_src_buff, __pyx_k_src_buff, sizeof(__pyx_k_src_buff), 0, 0, 1, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
  {&__pyx_n_s_zlib, __pyx_k_zlib, sizeof(__pyx_k_zlib), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 136, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "bgzip_utils.pyx":237
 *         assert NULL != out
 *     else:
 *         raise Exception("dst_buff_obj must be a memoryview instance.")             # <<<<<<<<<<<<<<
 *         # TODO: support bytearray objects
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_dst_buff_obj_must_be_a_memoryvie); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "bgzip_utils.pyx":178
 * 
 * 
 * def decompress_into(bytes src_buff, bytearray dst_buff, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`, resizing `dst_buff` as needed.
 */
  __pyx_tuple__2 = PyTuple_Pack(12, __pyx_n_s_src_buff, __pyx_n_s_dst_buff, __pyx_n_s_num_threads, __pyx_n_s_i, __pyx_n_s_dst, __pyx_n_s_out, __pyx_n_s_offset, __pyx_n_s_bytes_read, __pyx_n_s_inflated_size, __pyx_n_s_number_of_blocks, __pyx_n_s_blocks, __pyx_n_s_src); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);
  __pyx_codeobj__3 = (PyObject*)__Pyx_PyCode_New(3, 0, 12, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__2, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_bgzip_utils_bgzip_utils_pyx, __pyx_n_s_decompress_into, 178, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__3)) __PYX_ERR(0, 178, __pyx_L1_error)

  /* "bgzip_utils.pyx":217
 * 
 * 
 * def decompress_into_2(bytes src_buff, object dst_buff_obj, unsigned int offset, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`
 */
  __pyx_tuple__4 = PyTuple_Pack(13, __pyx_n_s_src_buff, __pyx_n_s_dst_buff_obj, __pyx_n_s_offset, __pyx_n_s_num_threads, __pyx_n_s_i, __pyx_n_s_out, __pyx_n_s_bytes_read, __pyx_n_s_bytes_inflated, __pyx_n_s_number_of_blocks, __pyx_n_s_blocks, __pyx_n_s_src, __pyx_n_s_dst_buff, __pyx_n_s_avail_out); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(4, 0, 13, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_bgzip_utils_bgzip_utils_pyx, __pyx_n_s_decompress_into_2, 217, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 217, __pyx_L1_error)

  /* "bgzip_utils.pyx":321
 * 
 * 
 * def compress_chunks(input_chunks, int num_threads):             # <<<<<<<<<<<<<<
 *     cdef int i, chunk_size, compressed_chunk_guess_size
 *     cdef int number_of_chunks = len(input_chunks)
 */
  __pyx_tuple__6 = PyTuple_Pack(12, __pyx_n_s_input_chunks, __pyx_n_s_num_threads, __pyx_n_s_i, __pyx_n_s_chunk_size, __pyx_n_s_compressed_chunk_guess_size, __pyx_n_s_number_of_chunks, __pyx_n_s_blocks, __pyx_n_s_chunk, __pyx_n_s_compressed_chunk, __pyx_n_s_chunks, __pyx_n_s_compressed_chunks, __pyx_n_s_ret); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(2, 0, 12, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_bgzip_utils_bgzip_utils_pyx, __pyx_n_s_compress_chunks, 321, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 321, __pyx_L1_error)

  /* "bgzip_utils.pyx":367
 *         raise Exception()
 * 
 * def compress_to_stream(input_buff_obj, list scratch_buffers, handle, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Compress the data in `input_buff_obj` and write it to `handle`.
 */
  __pyx_tuple__8 = PyTuple_Pack(12, __pyx_n_s_input_buff_obj, __pyx_n_s_scratch_buffers, __pyx_n_s_handle, __pyx_n_s_num_threads, __pyx_n_s_i, __pyx_n_s_chunk_size, __pyx_n_s_bytes_available, __pyx_n_s_number_of_chunks, __pyx_n_s_blocks, __pyx_n_s_compressed_chunks, __pyx_n_s_compressed_chunk, __pyx_n_s_input_view); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(4, 0, 12, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_bgzip_utils_bgzip_utils_pyx, __pyx_n_s_compress_to_stream, 367, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  /* InitThreads.init */
  #ifdef WITH_THREAD
PyEval_InitThreads();
#endif

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_11bgzip_utils_HEADER = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initbgzip_utils(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initbgzip_utils(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_bgzip_utils(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_bgzip_utils(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_bgzip_utils(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'bgzip_utils' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_bgzip_utils(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("bgzip_utils", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_bgzip_utils) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "bgzip_utils")) {
      if (unlikely(PyDict_SetItemString(modules, "bgzip_utils", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) goto __pyx_L1_error;
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) goto __pyx_L1_error;
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "bgzip_utils.pyx":1
 * import io             # <<<<<<<<<<<<<<
 * import zlib
 * import struct
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_io, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_io, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "bgzip_utils.pyx":2
 * import io
 * import zlib             # <<<<<<<<<<<<<<
 * import struct
 * from math import ceil
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_zlib, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_zlib, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "bgzip_utils.pyx":3
 * import io
 * import zlib
 * import struct             # <<<<<<<<<<<<<<
 * from math import ceil
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struct, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "bgzip_utils.pyx":4
 * import zlib
 * import struct
 * from math import ceil             # <<<<<<<<<<<<<<
 * 
 * from libc.stdlib cimport abort
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ceil);
  __Pyx_GIVEREF(__pyx_n_s_ceil);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ceil);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_math, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ceil); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ceil, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":20
 * 
 * 
 * cdef const unsigned char * MAGIC = "\037\213\010\4"             # <<<<<<<<<<<<<<
 * # cdef Bytef * HEADER = b"\037\213\010\4\0\0\0\0\0\377\6\0\102\103\2\0"
 * cdef bytes HEADER = b"\037\213\010\4\0\0\0\0\0\377\6\0\102\103\2\0"
 */
  __pyx_v_11bgzip_utils_MAGIC = ((unsigned char const *)((char const *)"\037\213\010\004"));

  /* "bgzip_utils.pyx":22
 * cdef const unsigned char * MAGIC = "\037\213\010\4"
 * # cdef Bytef * HEADER = b"\037\213\010\4\0\0\0\0\0\377\6\0\102\103\2\0"
 * cdef bytes HEADER = b"\037\213\010\4\0\0\0\0\0\377\6\0\102\103\2\0"             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(__pyx_kp_b_BC);
  __Pyx_XGOTREF(__pyx_v_11bgzip_utils_HEADER);
  __Pyx_DECREF_SET(__pyx_v_11bgzip_utils_HEADER, __pyx_kp_b_BC);
  __Pyx_GIVEREF(__pyx_kp_b_BC);

  /* "bgzip_utils.pyx":178
 * 
 * 
 * def decompress_into(bytes src_buff, bytearray dst_buff, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`, resizing `dst_buff` as needed.
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_11bgzip_utils_1decompress_into, NULL, __pyx_n_s_bgzip_utils); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_decompress_into, __pyx_t_2) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":217
 * 
 * 
 * def decompress_into_2(bytes src_buff, object dst_buff_obj, unsigned int offset, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Inflate bytes from `src_buff` into `dst_buff`
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_11bgzip_utils_3decompress_into_2, NULL, __pyx_n_s_bgzip_utils); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_decompress_into_2, __pyx_t_2) < 0) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":315
 * 
 * 
 * cdef unsigned int _block_inflated_size = 65280             # <<<<<<<<<<<<<<
 * cdef unsigned int _block_metadata_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + sizeof(BlockTailer)
 * block_inflated_size = _block_inflated_size
 */
  __pyx_v_11bgzip_utils__block_inflated_size = 0xFF00;

  /* "bgzip_utils.pyx":316
 * 
 * cdef unsigned int _block_inflated_size = 65280
 * cdef unsigned int _block_metadata_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + sizeof(BlockTailer)             # <<<<<<<<<<<<<<
 * block_inflated_size = _block_inflated_size
 * block_metadata_size = _block_metadata_size
 */
  __pyx_v_11bgzip_utils__block_metadata_size = (((sizeof(__pyx_t_11bgzip_utils_BlockHeader)) + (sizeof(__pyx_t_11bgzip_utils_BlockHeaderBGZipSubfield))) + (sizeof(__pyx_t_11bgzip_utils_BlockTailer)));

  /* "bgzip_utils.pyx":317
 * cdef unsigned int _block_inflated_size = 65280
 * cdef unsigned int _block_metadata_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + sizeof(BlockTailer)
 * block_inflated_size = _block_inflated_size             # <<<<<<<<<<<<<<
 * block_metadata_size = _block_metadata_size
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_11bgzip_utils__block_inflated_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_block_inflated_size, __pyx_t_2) < 0) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":318
 * cdef unsigned int _block_metadata_size = sizeof(BlockHeader) + sizeof(BlockHeaderBGZipSubfield) + sizeof(BlockTailer)
 * block_inflated_size = _block_inflated_size
 * block_metadata_size = _block_metadata_size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_11bgzip_utils__block_metadata_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_block_metadata_size, __pyx_t_2) < 0) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":321
 * 
 * 
 * def compress_chunks(input_chunks, int num_threads):             # <<<<<<<<<<<<<<
 *     cdef int i, chunk_size, compressed_chunk_guess_size
 *     cdef int number_of_chunks = len(input_chunks)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_11bgzip_utils_5compress_chunks, NULL, __pyx_n_s_bgzip_utils); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compress_chunks, __pyx_t_2) < 0) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":367
 *         raise Exception()
 * 
 * def compress_to_stream(input_buff_obj, list scratch_buffers, handle, int num_threads):             # <<<<<<<<<<<<<<
 *     """
 *     Compress the data in `input_buff_obj` and write it to `handle`.
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_11bgzip_utils_7compress_to_stream, NULL, __pyx_n_s_bgzip_utils); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compress_to_stream, __pyx_t_2) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bgzip_utils.pyx":1
 * import io             # <<<<<<<<<<<<<<
 * import zlib
 * import struct
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init bgzip_utils", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init bgzip_utils");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned short    uint16_t;
        #else
           typedef unsigned __int16  uint16_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define GCC_DIAGNOSTIC
#endif
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            *(uint16_t*)dpos = ((const uint16_t*)DIGIT_PAIRS_8)[digit_pos];
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            *(uint16_t*)dpos = ((const uint16_t*)DIGIT_PAIRS_10)[digit_pos];
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    if (last_one_off) {
        assert(*dpos == '0');
        dpos++;
    }
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
    const unsigned int neg_one = (unsigned int) ((unsigned int) 0 - (unsigned int) 1), const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* Print */
#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
    const unsigned int neg_one = (unsigned int) ((unsigned int) 0 - (unsigned int) 1), const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* PrintOne */
#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
